Package: CNORode
Type: Package
Title: ODE add-on to CellNOptR
Version: 1.52.0
Date: 2022-03-22
Authors@R: c(
	person(given = "David",
           family = "Henriques",
           role = "aut"),
    person(given = "Thomas",
           family = "Cokelaer",
           role = c("aut")),
    person(given = "Federica",
           family = "Eduati",
           role = "ctb"),
    person(given = "Enio",
           family = "Gjerga",
           role = "ctb"),
    person(given = "Attila",
           family = "Gabor",
           role = "cre",
           email = "attila.gabor@uni-heidelberg.de")
           )
Author: David Henriques, Thomas Cokelaer, Attila Gabor, Federica Eduati, Enio Gjerga
Description: Logic based ordinary differential equation (ODE) add-on to
    CellNOptR.
License: GPL-2
LazyLoad: yes
Depends: CellNOptR, genalg
Enhances: doParallel, foreach
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
biocViews: ImmunoOncology, CellBasedAssays, CellBiology, Proteomics,
        Bioinformatics, TimeCourse
RoxygenNote: 7.1.2
git_url: https://git.bioconductor.org/packages/CNORode
git_branch: RELEASE_3_22
git_last_commit: 0bfa1d4
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: yes
Packaged: 2025-10-30 03:12:32 UTC; biocbuild
Maintainer: Attila Gabor <attila.gabor@uni-heidelberg.de>

# Generated by roxygen2: do not edit by hand

export(createLBodeContPars)
export(defaultParametersGA)
export(defaultParametersSSm)
export(getLBodeContObjFunction)
export(getLBodeDataSim)
export(getLBodeMINLPObjFunction)
export(getLBodeModelSim)
export(getLBodeSimFunction)
export(getStates)
export(incidence2Adjacency)
export(minlpLBodeSSm)
export(parEstimationLBode)
export(parEstimationLBodeGA)
export(parEstimationLBodeSSm)
export(plotLBodeFitness)
export(plotLBodeModelSim)
export(runCNORode)
export(simdata2cnolist)
export(simulate)
import(CellNOptR)
importFrom(genalg,rbga)
useDynLib(CNORode,sim_logic_ode)

# CNORode: a logic based ordinary differential equation add-on for CellNOptR

This version of CNORode is a continuation of the [CNORode2017](https://github.com/saezlab/CNORode2017) package. CNORode2017 was lamor equivalent to CNORode with some added features, such as the sparsity enforcing
regularisation and parameter uncertainty analysis based on bootstrapping. 
Maintaining two packages are time consuming, therefore we decided to merge the packages and continue with CNORode2017.

The older version of CNORode can be downloaded from [release_v1.23](https://github.com/saezlab/CNORode/releases/tag/v1.23.0)

## Where to start
We refer to the [*vignette*](https://www.bioconductor.org/packages/release/bioc/vignettes/CNORode/inst/doc/CNORode-vignette.pdf)
for learning usage of the classic CNORode. 

The added features of the CNORode2017 are summarised on the main site of [CNORode2017](https://github.com/saezlab/CNORode2017). 

For more general information about the CellNOpt project visit: http://www.cellnopt.org/.


### How to install

*CNORode* requires *CellNOptR* and *MEIGOR* which are available in Bioconductor and can be installed typing:

```R
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("CNORode")
BiocManager::install("MEIGOR")
```

Alternatively, you can obtain the most recent version of *CellNOptR* from our GitHub repository:

```R
remotes::install_github("saezlab/CellNOptR")
```

*CNORode* can be installed from this repository by typing:

```R
remotes::install_github"saezlab/CNORode")
```



### Basic info on logic based ODE and CNORode
For an introduction on logic based ODE we recommend reading the original publication [(Wittmann et al., BMC Syst Biol., 2009)](https://www.ncbi.nlm.nih.gov/pmc/articles/PMC2764636/). Briefly, logic based ODE are ordinary differential equations (ODE) derived from logic rules using continuous update function (*B<sub>i</sub>*), which allows to have a continuous description of the behaviour of the species of interest both in time and in state. Each species *x<sub>i</sub>* is described by an ODE:

*d(x<sub>i</sub>)/dt=&tau;<sub>i</sub>(B<sub>i</sub>(f(x<sub>i1</sub>), f(x<sub>i2</sub>), ..., f(x<sub>iN</sub>))-x<sub>i</sub>)*

where *&tau;<sub>i</sub>* is the life-time of the species and *x<sub>i1</sub>, x<sub>i2</sub>, ... x<sub>iN</sub>* are its *N* regulators. Each regulation is described by a transfer function *f(x<sub>ij</sub>)* which can be, for example, a linear relationsip or a sigmoidal (Hill like) curve.

### New features of *CNORode2017*
Please visit [CNORode2017](https://github.com/saezlab/CNORode2017). 

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2013 - EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
#' @export
defaultParametersSSm <-
function(){

    params = list()
    params$maxeval=Inf
    params$maxtime=100
    params$ndiverse=NULL
    params$dim_refset=NULL
    params$local_solver=NULL
	  params$verbose = 0
    params$transfer_function = 3
    params$reltol = 1e-04
    params$atol = 0.001
	  params$maxStepSize = Inf
    params$maxNumSteps = 1e+05
    params$maxErrTestsFails = 50
    params$nan_fac = 1
    params$time = 1 
    
    # added for Eduati et al. 2017 paper (with default parameters should be same as original CNORode)
    params$lambda_tau=0
    params$lambda_k=0
    params$bootstrap=F
    params$SSpenalty_fac=0
    params$SScontrolPenalty_fac=0
    params$boot_seed=sample(1:10000,1)

    return(params)
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2013 - EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
#' @export
minlpLBodeSSm <-
function
(
    cnolist,                model,                    ode_parameters=NULL,
    int_x0=NULL,            indices=NULL,            maxeval=Inf,
    maxtime=100,            ndiverse=NULL,            dim_refset=NULL,
    local_solver=NULL,      time=1,                    verbose=0,
    transfer_function=3,    reltol=1e-4,            atol=1e-3,
    maxStepSize=Inf,        maxNumSteps=100000,        maxErrTestsFails=50,
    nan_fac=1
)
{

    if (class(cnolist)=="CNOlist"){cnolist = compatCNOlist(cnolist)}
    adjMat=incidence2Adjacency(model);
    if(is.null(ode_parameters))
    {
        ode_parameters=createLBodeContPars(model,random=TRUE);

    }

    ode_parameters$model=model;
    #Check if essR is installed
    dummy_f<-function(x){
        return(0);
    }
    problem<-list(f=dummy_f,x_L=rep(0),x_U=c(1));
    opts<-list();
    opts$maxeval=0;
    opts$maxtime=0;

    val=tryCatch({essR(problem,opts)}, error=function(e){print("essR package not found.
    SSm not available. Install the package and load it or try the Genetic Algorithm
    optimiser instead.");return(ode_parameters);});
    ######################################################################################

    n_cont=length(ode_parameters$index_opt_pars);
    n_int=dim(model$interMat)[2];

    if(is.null(int_x0)){
        int_x0=as.integer(round(runif(n_int)));
    }

    problem=list();
    problem$f<-getLBodeMINLPObjFunction(cnolist,model,ode_parameters,indices,time,
            verbose,transfer_function,reltol,atol,maxStepSize,maxNumSteps,maxErrTestsFails);
    problem$x_L <- c(ode_parameters$LB[ode_parameters$index_opt_pars],matrix(0,1,n_int));
    problem$x_U <- c(ode_parameters$UB[ode_parameters$index_opt_pars],matrix(1,1,n_int));
    problem$x_0<- c(ode_parameters$parValues[ode_parameters$index_opt_pars],int_x0);
    problem$int_var=0;

    problem$bin_var=n_int;
    opts=list();
    opts$maxeval=maxeval;
    opts$maxtime=maxtime;
    if(!is.null(ndiverse))opts$ndiverse=ndiverse;
    if(!is.null(dim_refset))opts$dim_refset=dim_refset;
    optimization_res=essR(problem,opts);

    ode_parameters$bitString=optimization_res$xbest[(n_cont+1):(n_cont+n_int)];
    index_reactions=which(as.logical(ode_parameters$bitString));

    model$reacID=model$reacID[index_reactions];
    model$interMat=model$interMat[,index_reactions];
    model$notMat=model$notMat[,index_reactions];

    ode_parameters$model=model;

    ode_parameters$parValues[ode_parameters$index_opt_pars]=optimization_res$xbest[1:n_cont];
    ode_parameters$ssm_results=optimization_res;
    return(ode_parameters);
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2013 - EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
#' @export
getStates <-
function(adjacency)
{
	nSpecies=dim(adjacency)[1];
	count=0;
	res=matrix(0,1,nSpecies);
	for(j in 1:nSpecies)
	{
		for(i in 1:nSpecies)
		{
			if(adjacency[i,j])
			{
				res[j]=1;
			}
		}
	}
	return(res);
}

#' Crossvalidate ODE model
#' 
#' k-fold crossvalidation for logic ODE model
#' 
#' Does a k-fold cross-validation for logic ODE CellNOpt models. In k-iterations a 
#' fraction of the data is eliminated from the CNOlist. The model is trained on the 
#' remaining data and then the model predicts the held-out data. Then the prediction
#' accuracy is reported for each iteration. 
#' 
#' @param CNOlist Cnolist which contains all the experiments  
#' @param model a model prepared for the training  
#' @param nfolds number of folds - default is 10. Although nfolds can be as large as the sample size (leave-one-out CV),
#'  it is not recommended for large datasets.  
#' @param foldid an optional vector of values between `1` and `nfold` identifying what fold each observation is in. If supplied, `nfold` can be missing.  
#' @param type define the way to do the crossvalidation. The default is 
#' `type="datapoint"`, which assigns the data randomly into folds. 
#' The option `type="experiment"` uses whole experiments for crossvalidation 
#' (all data corresponding to a cue combination). The `type=observable` uses the
#' subset of nodes across all experiments for crossvalidation.  
#' @param method Selection of optimization method: only "ga" or "essm" arguments are accepted
#' @param parallel use for parallel execution, requires the doParallel package
#' @param ode_parameters list of fitted logic ODE parameter
#' @param paramsSSm parameters for the SSm optimizer for running the optimization in crossvalidation 
#' @seealso \code{\link{parEstimationLBode}}  

crossvalidateODE = function(CNOlist, model, nfolds=10, foldid=NULL, type='datapoint', parallel=FALSE, ode_parameters=NULL, paramsSSm=NULL, method = "essm"){
  
  if ((class(CNOlist)=="CNOlist")==FALSE){
    CNOlist = CellNOptR::CNOlist(CNOlist)
  }
  
  crossvalidate.call = match.call(expand.dots = TRUE)
  
  if(!(type%in%c('datapoint','experiment','observable'))){
    
    error("Wrong argument for type. It should either be 'datapoint', 'experiment' or 'observable'")
    
  } else {
    
    if(type=='datapoint'){
      N = prod(dim(CNOlist@signals[[1]]))
    }else if(type=='experiment'){
      N = nrow(CNOlist@signals[[1]])
    }else if(type=="observable"){
      N = ncol(CNOlist@signals[[1]])
    }
    
  }
  
  if (is.null(foldid)){
    
    foldid = sample(rep(seq(nfolds), length = N))
    
  } else {
    
    nfolds = max(foldid)
    
  }
  
  outlist = as.list(seq(nfolds))
  
  if (parallel) {
    if (!requireNamespace("doParallel", quietly = TRUE)) {
      stop("Package \"doParallel\" needed for this function to work. Please install it or set parallel = FALSE.",
           call. = FALSE)
    }
    
    `%dopar%` <- foreach::`%dopar%`
    outlist = foreach::foreach(i = seq(nfolds), .packages = c("CNORode")) %dopar% 
    {
      whichI = foldid == i
      CNOlist.sub = CNOlist
      CNOlist.cv = CNOlist
      if(type=='datapoint'){
        for(timeIndex in 1:length(CNOlist@timepoints)){
          CNOlist.sub@signals[[timeIndex]][whichI] = NA
          CNOlist.cv@signals[[timeIndex]][!whichI] = NA
        }
      }else if(type=='experiment'){
        for(timeIndex in 1:length(CNOlist@timepoints)){
          CNOlist.sub@signals[[timeIndex]][whichI,] = NA
          CNOlist.cv@signals[[timeIndex]][!whichI,] = NA
        }
      }else if(type=="observable"){
        for(timeIndex in 1:length(CNOlist@timepoints)){
          CNOlist.sub@signals[[timeIndex]][,whichI] = NA
          CNOlist.cv@signals[[timeIndex]][,!whichI] = NA
        }
      }
      
      if(is.null(ode_parameters)){
        
        ode_parameters=createLBodeContPars(model, LB_n = 1, LB_k = 0,
                                           LB_tau = 0, UB_n = 4, UB_k = 1, UB_tau = 1, default_n = 3,
                                           default_k = 0.5, default_tau = 0.01, opt_n = FALSE, opt_k = TRUE,
                                           opt_tau = TRUE, random = TRUE)
        
      }
      
      if(is.null(paramsSSm)){
        
        paramsSSm=defaultParametersSSm()
        paramsSSm$local_solver = "DHC"
        paramsSSm$maxtime = 600;
        paramsSSm$maxeval = Inf;
        paramsSSm$atol=1e-6;
        paramsSSm$reltol=1e-6;
        paramsSSm$nan_fac=0;
        paramsSSm$dim_refset=30;
        paramsSSm$n_diverse=1000;
        paramsSSm$maxStepSize=Inf;
        paramsSSm$maxNumSteps=10000;
        transferFun=4;
        paramsSSm$transfer_function = transferFun;
        
        paramsSSm$lambda_tau=0
        paramsSSm$lambda_k=0
        paramsSSm$bootstrap=F
        paramsSSm$SSpenalty_fac=10
        paramsSSm$SScontrolPenalty_fac=1000
        
      }
      
      outlist = list()
      res = parEstimationLBode(cnolist = CNOlist.sub, model = model, ode_parameters = ode_parameters, method = method, paramsSSm = paramsSSm)
      outlist$fit = res
    
      outlist$cvScore = res$ssm_results$fbest
      outlist
    }
    
  }
  else {
    cat("fold \t fitScore \t cvScore \t nTolModels \t bString\n")
    
    for (i in seq(nfolds)) {
      whichI = foldid == i
      CNOlist.sub = CNOlist
      CNOlist.cv = CNOlist
      if(type=='datapoint'){
        for(timeIndex in 1:length(CNOlist@timepoints)){
          CNOlist.sub@signals[[timeIndex]][whichI] = NA
          CNOlist.cv@signals[[timeIndex]][!whichI] = NA
        }
      }else if(type=='experiment'){
        for(timeIndex in 1:length(CNOlist@timepoints)){
          CNOlist.sub@signals[[timeIndex]][whichI,] = NA
          CNOlist.cv@signals[[timeIndex]][!whichI,] = NA
        }
      }else if(type=="observable"){
        for(timeIndex in 1:length(CNOlist@timepoints)){
          CNOlist.sub@signals[[timeIndex]][,whichI] = NA
          CNOlist.cv@signals[[timeIndex]][,!whichI] = NA
        }
      }
      
      if(is.null(ode_parameters)){
        
        ode_parameters=createLBodeContPars(model, LB_n = 1, LB_k = 0,
                                           LB_tau = 0, UB_n = 4, UB_k = 1, UB_tau = 1, default_n = 3,
                                           default_k = 0.5, default_tau = 0.01, opt_n = FALSE, opt_k = TRUE,
                                           opt_tau = TRUE, random = TRUE)
        
      }
      
      if(is.null(paramsSSm)){
        
        paramsSSm=defaultParametersSSm()
        paramsSSm$local_solver = "DHC"
        paramsSSm$maxtime = 600;
        paramsSSm$maxeval = Inf;
        paramsSSm$atol=1e-6;
        paramsSSm$reltol=1e-6;
        paramsSSm$nan_fac=1000;
        paramsSSm$dim_refset=30;
        paramsSSm$n_diverse=1000;
        paramsSSm$maxStepSize=Inf;
        paramsSSm$maxNumSteps=10000;
        transferFun=4;
        paramsSSm$transfer_function = transferFun;
        
        paramsSSm$lambda_tau=0
        paramsSSm$lambda_k=0
        paramsSSm$bootstrap=F
        paramsSSm$SSpenalty_fac=10
        paramsSSm$SScontrolPenalty_fac=1000
        
      }
      
      res = parEstimationLBode(cnolist = CNOlist.sub, model = model, ode_parameters = ode_parameters, method = method, paramsSSm = paramsSSm)
      
      outlist[[i]] = list()
      outlist[[i]]$fit = res
      
      outlist[[i]]$cvScore = res$ssm_results$fbest
      
    }
    
  }
  
  return(outlist)
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2013 - EMBL-EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
#' @export
simdata2cnolist <- function(sim_data, cnolist, model){ 

    if (class(cnolist)=="CNOlist"){cnolist = compatCNOlist(cnolist)}
    adjMatrix=incidence2Adjacency(model);
    states_index=which(as.logical(getStates(adjMatrix)));

    sim_data=lapply(sim_data,function(x) x[,states_index, drop=F]);

    times=cnolist$timeSignals;
    cnolist$valueSignals=sim_data;
 
    cnolist$valueVariances = sim_data # create the data structure
    ## set values to NA
    for (i in seq_along(cnolist$valueVariances)){
        cnolist$valueVariances[[i]][cnolist$valueVariances[[i]]>0] <- NA
    }

    cnolist$namesSignals = model$namesSpecies[states_index];
    cnolist$namesCues = c(cnolist$namesStimuli,cnolist$namesInhibitors);
    cnolist$valueCues = cbind(cnolist$valueStimuli,cnolist$valueInhibitors);
    cnolist$valueCues = as.matrix(cnolist$valueCues);
    cnolist$valueCues[which(cnolist$valueCues>0)]=1;
    colnames(cnolist$valueCues)=cnolist$namesCues;

    return(CNOlist(cnolist))
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2013 - EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
#' @export
getLBodeMINLPObjFunction <-function
(
        cnolist,                model,                    ode_parameters,
        indices=NULL,            time=1,                    verbose=0,
        transfer_function=3,    reltol=1e-4,            atol=1e-3,
        maxStepSize=Inf,        maxNumSteps=100000,        maxErrTestsFails=50,
        nan_fac=1
)
{
    if (class(cnolist)=="CNOlist"){cnolist = compatCNOlist(cnolist)}

    adjMatrix=incidence2Adjacency(model);

    if(is.null(indices))indices=indexFinder(cnolist,model,verbose=FALSE);

    n_cont=length(ode_parameters$index_opt_pars);
    n_int=dim(model$interMat)[2];

    sim_function<-getLBodeSimFunction(cnolist,model,adjMatrix1=adjMatrix,
            indices1=indices,odeParameters1=ode_parameters$parValues, time1=time,verbose1=verbose,
            transfer_function1=transfer_function,reltol1=reltol,atol1=atol,maxStepSize1=maxStepSize,
            maxNumSteps1=maxNumSteps,maxErrTestsFails1=maxErrTestsFails)

    logic_ode_MINLP_objective_function<-function
    (
            x,                        cnolist1=cnolist,                    model1=model,
            adjMatrix1=adjMatrix,    n_cont1=n_cont,                        n_int1=n_int,
            indices1=indices,        ode_parameters1=ode_parameters,        sim_function1=sim_function,
            nan_fac1=nan_fac
    )
    {
        temp_model=model1;
        x_cont=as.double(x[seq(1,n_cont1)]);
        x_int=as.integer(x[seq((n_cont1+1),(n_cont1+n_int1))]);
        if(length(which(as.logical(x_int)))<2)return(1);
        temp_model$interMat=model1$interMat[,which(as.logical(x_int))];
        temp_model$notMat=model1$notMat[,which(as.logical(x_int))];
        ode_parameters1$parValues[ode_parameters1$index_opt_pars]=x_cont;
        sim=sim_function1(cnolist1,temp_model,ode_parameters1$parValues);
                sim<-as.vector(unlist(lapply(sim,function(x) x[,indices1$signals])));
        measured_values<-as.vector(unlist(lapply(cnolist1$valueSignals,function(x)x)));
        NaNs_sim=which(is.na(sim));
        not_NaNs_data=which(!is.na(measured_values));
        not_NaNs_sim=which(!is.na(sim));
        not_NaNs=intersect(not_NaNs_sim,not_NaNs_data);
        NaNs=intersect(not_NaNs_data,NaNs_sim);
        error=sum((sim[not_NaNs]-measured_values[not_NaNs])^2);
        res=(error+length(NaNs)*nan_fac1)/length(not_NaNs_data);
        return(res);
    }
    return(logic_ode_MINLP_objective_function);
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2013 - EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
#' @export
plotLBodeModelSim <-function
(
	cnolist,				    model,					    ode_parameters=NULL,
	indices=NULL,			    adjMatrix=NULL,			  	timeSignals=NULL,
	time=1,						verbose=0, 				    transfer_function=3,		
	reltol=1e-4,				atol=1e-3,				    maxStepSize=Inf,
	maxNumSteps=100000,			maxErrTestsFails=50,  		large=FALSE,          		
	nsplit=4, show=TRUE
)
{
	
	if (class(cnolist)=="CNOlist"){cnolist = compatCNOlist(cnolist)}
	if(is.null(indices))indices=indexFinder(cnolist,model);
	if(is.null(adjMatrix))adjMatrix=incidence2Adjacency(model);
	if(is.null(ode_parameters))ode_parameters=createLBodeContPars(model);
	if(!is.null(timeSignals))cnolist$timeSignals=timeSignals;
	
	
	sim_data=getLBodeModelSim(cnolist,model,
							  ode_parameters,indices,timeSignals,time,verbose,transfer_function,
							  reltol,atol,maxStepSize,maxNumSteps,maxErrTestsFails);
	
	temp=list();
	
	for(i in 1:length(cnolist$timeSignals)){
		
		temp[[i]]=sim_data[[i]];
	}
	
	sim_data=temp;
	
	cnolist = simdata2cnolist(sim_data, cnolist, model)
	
	if (show==T){
		if(large)
		{
			plotCNOlistLarge(cnolist,nsplit);
		}
		else
		{   
			plotCNOlist(cnolist);
		}
	}
	
	return(sim_data);
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2013 - EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
#' @export
defaultParametersGA <-
 function(){

    params = list()
    params$mutationChance=NA
    params$popSize=200
    params$iters=100
    params$elitism=NA
    params$time = 1
    params$monitor=TRUE
    params$verbose = 0
    params$transfer_function = 3
    params$reltol = 1e-04
    params$atol = 0.001
    params$maxStepSize = Inf 
    params$maxNumSteps = 1e+05
    params$maxErrTestsFails = 50
    params$nan_fac = 1

    return(params)
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2013 - EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
# $Id: incidence2Adjacency.R 3157 2013-01-09 16:04:09Z cokelaer $
#' @export
incidence2Adjacency <-
function(model)
{ 
	incidence=model$interMat;
	nNodes=dim(incidence)[1];
	nEdges=dim(incidence)[2];
	adjacency=matrix(0,nNodes,nNodes);
	
	for(j in 1:nEdges)
	{
		for(i in 1:nNodes)
		{
			if(incidence[i,j]==1)
			{
				node1=i;
				for(k in 1:nNodes)
				{
					if(incidence[k,j]==-1)
					{
						adjacency[k,node1]=1
					}
				}
			}
		}
	}
	return(adjacency);  
}

#' @title runCNORode
#'
#' @description A one-line wrapper of the CNORode pipeline
#'
#' @param model A filename of prior knowledge network (PKN) in the SIF format
#' @param data A measurement filename in the MIDAS format
#' @param results_folder  results folder for the analysis. 
#' @param compression compress the prior knowledge network (TRUE), see \code{\link{preprocessing}}
#' @param cutNONC cut non-observable non-controllable node from PKN (TRUE), see \code{\link{preprocessing}}
#' @param expansion expand OR gates in the PKN (FALSE), see \code{\link{preprocessing}}
#' @param LB_n lower bound on parameter n, see \code{\link{createLBodeContPars}}
#' @param LB_k lower bound on parameter k, see \code{\link{createLBodeContPars}}
#' @param LB_tau lower bound on parameter tau, see \code{\link{createLBodeContPars}}
#' @param UB_n upper bound on parameter n, see \code{\link{createLBodeContPars}}
#' @param UB_k upper bound on parameter k, see \code{\link{createLBodeContPars}}
#' @param UB_tau upper bound on parameter tau, see \code{\link{createLBodeContPars}}
#' @param default_n default value of parameter n, see \code{\link{createLBodeContPars}}
#' @param default_k default value of parameter k, see \code{\link{createLBodeContPars}}
#' @param default_tau default value of parameter tau, see \code{\link{createLBodeContPars}}
#' @param opt_n should parameter n be optimised, see \code{\link{createLBodeContPars}}
#' @param opt_k should parameter k be optimised, see \code{\link{createLBodeContPars}}
#' @param opt_tau should parameter tau be optimised, see \code{\link{createLBodeContPars}}
#' @param random initial parameter vector generation (TRUE: random, FALSE: half of the LB-UB)
#' @param maxeval maximum number of funciton evaluations in the optimisation, see \code{\link{parEstimationLBodeSSm}}
#' @param maxtime maximum CPU time (in seconds) spent on optimisation before calling final refinement, see \code{\link{parEstimationLBodeSSm}}
#' @param transfer_function trandfer function types represented by the edges, see \code{\link{parEstimationLBodeSSm}}
#' @param nan_fac penalty for NA simulations, see \code{\link{parEstimationLBodeSSm}}
#' @param lambda_tau regularisation penalty for tau parameters, see \code{\link{parEstimationLBodeSSm}}
#' @param lambda_k regularisation penalty for k parameters for optimisation, see \code{\link{parEstimationLBodeSSm}}
#' 
#' @examples
#' \dontrun{
#' model = system.file("extdata", "ToyModelMMB_FeedbackAnd.sif",package="CNORode")
#' data = system.file("extdata", "ToyModelMMB_FeedbackAnd.csv", package="CNORode")
#' res = runCNORode(model,data,results_folder = "./results")
#' }
#' @import CellNOptR
#' @export

runCNORode <- function(model,
					  data,
					  compression=TRUE,
					  results_folder = "CNORode_results",
					  cutNONC=TRUE,
					  expansion=FALSE,     
					  LB_n = 1,
					  LB_k = 0.1,
					  LB_tau = 0.01,
					  UB_n = 5,
					  UB_k = 0.9,
					  UB_tau = 10,
					  default_n = 3,
					  default_k = 0.5,
					  default_tau = 1,
					  opt_n = TRUE,
					  opt_k = TRUE,
					  opt_tau = TRUE,
					  random = TRUE,
					  maxeval = 1e5,
					  maxtime = 60,
					  transfer_function = 3,
					  nan_fac = 1,
					  lambda_tau = 0,
					  lambda_k = 0
) {
	
	
	# Manual assignment of network (PKN) and experimental data (MIDAS) files
	pknmodel <- readSIF(model) # build a prior-knowledge network from SIF file
	cnolist <- CNOlist(data) # import experimental data from MIDAS file
	
	if(!dir.exists(results_folder)) dir.create(results_folder)
	
	pdf(file.path(results_folder,"1_network_original.pdf"))
	plotModel(pknmodel,CNOlist = cnolist)
	dev.off()
	
	
	# Now use CNORode specific pre-processing (expansion=FALSE by default and report)
	model <- preprocessing(data=cnolist,model=pknmodel, expansion=expansion, # model in CNORode shouldn't be expanded by pre-processing step
									   compression=compression, cutNONC=cutNONC,
									   verbose=FALSE) 
	
	pdf(file.path(results_folder,"2_network_preprocessed.pdf"))
	plotModel(model,CNOlist = cnolist)
	dev.off()
	
	# generate parameterised model from model object
	init_ode_parameters = createLBodeContPars(model, LB_n = 1, LB_k = 0.1,
										   LB_tau = 0.01, UB_n = 5, UB_k = 0.9, UB_tau = 10, default_n = 3,
										   default_k = 0.5, default_tau = 1, opt_n = TRUE, opt_k = TRUE,
										   opt_tau = TRUE, random = FALSE)

	
	# fit the model to data
	fit_result = parEstimationLBodeSSm(cnolist = cnolist,
									model = model,
									ode_parameters = init_ode_parameters,
									maxeval = maxeval,
									maxtime = maxtime,
									local_solver = "DHC",
									transfer_function = transfer_function,
									nan_fac = nan_fac,
									lambda_k = lambda_k,
									lambda_tau = lambda_tau
									)
	
	# report results
	pdf(file.path(results_folder,"3_model_fittness.pdf"))
	simulatedData=plotLBodeFitness(cnolist = cnolist, 
								   model = model,
								   ode_parameters = fit_result,
								   transfer_function = transfer_function)
	
	dev.off()
	
	
	# collect results 
	output = list()
	output$model = model
	output$cnolist = cnolist
	output$init_ode_parameters = init_ode_parameters
	output$fit_results = fit_result
	output$model_siulation = simulatedData
	
	
	save(output, file = file.path(results_folder,"CNORode_PostHocResults.Rdata"))
	
	return(output)
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2013 - EMBL-EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
#' @export

simulate <-function(cnolist,model,ode_parameters=NULL,
        indices=NULL, adjMatrix=NULL, time=1, verbose=0, transfer_function=3, 
        reltol=1e-4, atol=1e-3, maxStepSize=Inf, maxNumSteps=100000,
        maxErrTestsFails=50)
{
    if (class(cnolist)=="CNOlist"){cnolist = compatCNOlist(cnolist)}
    if(is.null(indices))
        indices=indexFinder(cnolist,model);
    if(is.null(adjMatrix))
        adjMatrix=incidence2Adjacency(model);
    if(is.null(ode_parameters))
        ode_parameters=createLBodeContPars(model);

    timeSignals=NULL;

    sim_data=getLBodeDataSim(cnolist,model,
            ode_parameters,indices,timeSignals,time,verbose,
            transfer_function,reltol,atol,maxStepSize,maxNumSteps,
            maxErrTestsFails);

    return(sim_data)
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2013 - EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
#' @export
#' @useDynLib CNORode sim_logic_ode
getLBodeSimFunction <-function
(
		cnolist1,				model1,					adjMatrix1,
		indices1, 				odeParameters1,			time1=1,
		verbose1=0,				transfer_function1=3,	reltol1=1e-4,	
		atol1=1e-3,				maxStepSize1=Inf,		maxNumSteps1=100000,
		maxErrTestsFails1=50
)
{
				
	simulate_logic_based_ode_model <- function
	(
			cnolist,	 						sif, 									odeParameters,
			indices=indices1,					adjMatrix=adjMatrix1, 					time=time1,
			verbose=verbose1, 					transfer_function=transfer_function1,	reltol=reltol1,
			atol=atol1,							maxStepSize=maxStepSize1,				maxNumSteps=maxNumSteps1,
			maxErrTestsFails=maxErrTestsFails1
	)
	{	 
		interMat <- as.integer(as.vector(t(sif$interMat)))
		notMat <- as.integer(as.vector(t(sif$notMat)))
		adjMatrix <- as.integer(as.vector(t(adjMatrix)))
		nRows <- as.integer(dim(sif$interMat)[1])
		
		nCols <- as.integer(dim(sif$interMat)[2])
		verbose=as.integer(verbose);
	
		nPars <- as.integer(length(odeParameters))
	
		# cnolist
		timeSignals <- as.double(cnolist$timeSignals)
		valueInhibitors <- as.double(t(cnolist$valueInhibitors))
		valueSignals <- as.double(t(cnolist$valueSignals[[time]]))
		valueStimuli <- as.double(t(cnolist$valueStimuli))
		nTimes = as.integer(length(cnolist$timeSignals))
		
		# [[1]] allows to access to the first object in the list and retrieve its dimensions
		nExperiments = as.integer(dim(cnolist$valueSignals[[time]]))
		
		#indices
		nSignals <- as.integer(length(indices$signals))
		
		indexSignals <- as.integer(as.vector(indices$signals))
		nStimuli <- as.integer(length(indices$stimulated))
		indexStimuli <- as.integer(as.vector(indices$stimulated))
		nInhibitors <- as.integer(length(indices$inhibited))
		indexInhibitors <- as.integer(as.vector(indices$inhibited))
		transfer_function=as.integer(transfer_function);
		reltol=as.double(reltol);
		atol=as.double(atol);
		maxStepSize=as.double(maxStepSize);
		maxNumSteps=as.integer(maxNumSteps);
		maxErrTestsFails=as.integer(maxErrTestsFails);
		break_at_1st_fail=as.integer(0);
		
		res = .Call("sim_logic_ode",interMat,notMat,adjMatrix,nRows,nCols,nPars,timeSignals,
				valueInhibitors,valueSignals,valueStimuli,nTimes,nExperiments,nSignals,
				indexSignals,nStimuli,indexStimuli,nInhibitors,indexInhibitors,odeParameters,
				verbose,transfer_function,reltol,atol,maxStepSize,maxNumSteps,maxErrTestsFails,
				break_at_1st_fail);
		
		res = lapply(res,function(x){
			colnames(x) = sif$namesSpecies
			return(x)
			})
		return(res);
	}
	return(simulate_logic_based_ode_model);
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2013 - EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
#' @export
parEstimationLBodeSSm <-function
(
		cnolist,				model,					ode_parameters=NULL,
		indices=NULL,			maxeval=Inf,			maxtime=100,			
		ndiverse=NULL,			dim_refset=NULL, 		local_solver=NULL,      
		time=1,					verbose=0, 				transfer_function=3,	
		reltol=1e-4,			atol=1e-3,				maxStepSize=Inf,		
		maxNumSteps=100000,		maxErrTestsFails=50,	nan_fac=1,
		lambda_tau=0, lambda_k=0, bootstrap=FALSE,
		SSpenalty_fac=0, SScontrolPenalty_fac=0, boot_seed=sample(1:10000,1)
)
{
	
	if (class(cnolist)=="CNOlist"){cnolist = compatCNOlist(cnolist)}
	if (!requireNamespace("MEIGOR", quietly = TRUE)) {
		stop("Package \"MEIGOR\" needed for SSm to work. Please install it or try the Genetic Algorithm
	optimiser instead. MEIGOR got depricated on Bioconductor in release 3.18, but you can download the latest
			 version from Github: https://github.com/jaegea/MEIGOR/tree/RELEASE_3_18 ",
			 call. = FALSE)
	}
	
	
	checkSignals(CNOlist=cnolist,model=model)
	
	adjMat=incidence2Adjacency(model);
	if(is.null(ode_parameters)){
		ode_parameters=createLBodeContPars(model,random=TRUE);
	}
	if(is.null(indices))indices <- indexFinder(cnolist,model,verbose=FALSE);
	
	#Check if essR is installed
	dummy_f<-function(x){
		return(0);
	}
	problem<-list(f=dummy_f,x_L=rep(0),x_U=c(1));
	opts<-list();
	opts$maxeval=0;
	opts$maxtime=0;

    val=MEIGOR::essR(problem,opts)

	problem=list();
	problem$f<-getLBodeContObjFunction(cnolist=cnolist,
	                                   model=model,
	                                   ode_parameters=ode_parameters,
	                                   indices=indices,
	                                   time=time,
	                                   verbose=verbose,
	                                   transfer_function=transfer_function,
	                                   reltol=reltol,
	                                   atol=atol,
	                                   maxStepSize=maxStepSize,
	                                   maxNumSteps=maxNumSteps,
	                                   maxErrTestsFails=maxErrTestsFails,
	                                   nan_fac=nan_fac,
	                                   lambda_tau=lambda_tau,
	                                   lambda_k=lambda_k,
	                                   bootstrap=bootstrap,
	                                   SSpenalty_fac=SSpenalty_fac,
	                                   SScontrolPenalty_fac=SScontrolPenalty_fac,
	                                   boot_seed=boot_seed);
	problem$x_L <- ode_parameters$LB[ode_parameters$index_opt_pars];
	problem$x_U <- ode_parameters$UB[ode_parameters$index_opt_pars];
	problem$x_0<- ode_parameters$parValues[ode_parameters$index_opt_pars];
	problem$int_var =0;
	problem$bin_var =0;
	opts=list();
	opts$maxeval=maxeval;
	opts$maxtime=maxtime;
	if(!is.null(local_solver))opts$local_solver=local_solver;
	if(!is.null(ndiverse))opts$ndiverse=ndiverse;      
	if(!is.null(dim_refset))opts$dim_refset=dim_refset;  
	results=MEIGOR::essR(problem,opts);
	ode_parameters$parValues[ode_parameters$index_opt_pars]=results$xbest;
	ode_parameters$ssm_results=results;
	return(ode_parameters);	
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2013 - EMBL-EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
#' @export


parEstimationLBode<-function (cnolist, model, method="ga",
    ode_parameters = NULL, indices = NULL, paramsGA=NULL, paramsSSm=NULL)
{


    if (class(cnolist)=="CNOlist"){cnolist = compatCNOlist(cnolist)}
    if (method == "essm"){
        if (is.null(paramsSSm)){
            paramsSSm = defaultParametersSSm()
        }
        ode_parameters = parEstimationLBodeSSm(cnolist, model,
            ode_parameters=ode_parameters, indices=indices,
            maxeval=paramsSSm$maxeval,
            maxtime=paramsSSm$maxtime,
            ndiverse=paramsSSm$ndiverse,
            dim_refset=paramsSSm$dim_refset,
            local_solver=paramsSSm$local_solver,
            time=paramsSSm$time,
            verbose=paramsSSm$verbose,
            transfer_function=paramsSSm$transfer_function,
            reltol=paramsSSm$reltol,
            atol=paramsSSm$atol,
            maxStepSize=paramsSSm$maxStepSize,
            maxNumSteps=paramsSSm$maxNumSteps,
            maxErrTestsFails=paramsSSm$maxErrTestsFails,
            nan_fac=paramsSSm$nan_fac,
            
            # added
            lambda_tau=paramsSSm$lambda_tau,
            lambda_k=paramsSSm$lambda_k,
            bootstrap=paramsSSm$bootstrap,
            SSpenalty_fac=paramsSSm$SSpenalty_fac,
            SScontrolPenalty_fac=paramsSSm$SScontrolPenalty_fac,
            boot_seed=paramsSSm$boot_seed
            )


    }
    else if(method=="ga"){
        if (is.null(paramsGA)){
            paramsGA = defaultParametersGA()
        }
        ode_parameters = parEstimationLBodeGA(cnolist, model,
            ode_parameters=ode_parameters,
            indices=indices,
            mutationChance=paramsGA$mutationChance,
            popSize=paramsGA$popSize,
            iters=paramsGA$iters,
            elitism=paramsGA$elitism,
            time=paramsGA$time,
            monitor=paramsGA$monitor,
            verbose=paramsGA$verbose,
            transfer_function=paramsGA$transfer_function,
            reltol=paramsGA$reltol,
            atol=paramsGA$atol,
            maxStepSize=paramsGA$maxStepSize,
            maxNumSteps=paramsGA$maxNumSteps,
            maxErrTestsFails=paramsGA$maxErrTestsFails,
            nan_fac=paramsGA$nan_fac)
    }
    else{
        stop ("method argument must be either 'ga' or 'essm'." )
    }
    return(ode_parameters)
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2013 - EMBL-EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
#' @export
#' @importFrom genalg rbga



parEstimationLBodeGA<-function (cnolist, model, ode_parameters = NULL, indices = NULL, 
    mutationChance=NA, popSize=200,iters=100,elitism=NA, time = 1,monitor=TRUE,
	verbose = 0, transfer_function = 3, reltol = 1e-04, atol = 0.001,
	maxStepSize = Inf, maxNumSteps = 1e+05, maxErrTestsFails = 50, nan_fac = 1) 
{

    if (class(cnolist)=="CNOlist"){cnolist = compatCNOlist(cnolist)}
	
	checkSignals(CNOlist=cnolist,model=model)
	
    adjMat = incidence2Adjacency(model)
    if (is.null(ode_parameters)) {
        ode_parameters = createLBodeContPars(model, random = TRUE)
    }
    if (is.null(indices)) 
        indices <- indexFinder(cnolist, model, verbose = FALSE)
    problem = list()
    f_obj <- getLBodeContObjFunction(cnolist, model, ode_parameters, 
        indices, time, verbose, transfer_function, reltol, atol, 
        maxStepSize, maxNumSteps, maxErrTestsFails)
    x_L <- ode_parameters$LB[ode_parameters$index_opt_pars]
    x_U <- ode_parameters$UB[ode_parameters$index_opt_pars]
    x_0 <- ode_parameters$parValues[ode_parameters$index_opt_pars]
    
    if(monitor){
            plot.new();
            monitor_func <- function(obj){
                # plot the population
                xlim = c(0, obj$iters);
                par(mfrow=c(1,1));
                nIter=obj$iters-length(which(is.na(obj$best)));
                popSize=obj$popSize;
                heading = paste("Number of Evaluations=",popSize*nIter);
                plot(obj$best,main=heading,xlab="Number of Iterations", ylab="Objective Fuction",col='red')
                lines(obj$best, xlim=xlim,col="blue");
                print(paste("Iteration:",nIter,"  Best_f:",min(obj$best,na.rm=TRUE),"  N_Evals:",popSize*nIter,sep="")); 
            }
    }
    else{
        monitor_func=NULL;
    }

    res=genalg::rbga(x_L, x_U,popSize=popSize, iters=iters,monitorFunc=monitor_func, evalFunc=f_obj,
     showSettings=FALSE, verbose=FALSE,elitism=elitism,mutationChance=mutationChance);

    best_individual_index=which(res$evaluations==min(res$evaluations));
    best_individual=res$population[best_individual_index,];

    ode_parameters$parValues[ode_parameters$index_opt_pars] = best_individual;
    ode_parameters$res=res;
	
	
    return(ode_parameters)
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2013 - EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
#' @export
getLBodeDataSim<-function
(
		cnolist,				    model,					ode_parameters=NULL,
		indices=NULL,			    timeSignals=NULL,		time=1,					
		verbose=0,					transfer_function=3,	reltol=1e-4,			
		atol=1e-3,					maxStepSize=Inf,		maxNumSteps=100000,		
		maxErrTestsFails=50
)
{

    if (class(cnolist)=="CNOlist"){
    	cnolist = compatCNOlist(cnolist)
    }
	
	adjMat=incidence2Adjacency(model)
	
	if(is.null(indices)){
		indices <- indexFinder(cnolist,model,verbose=FALSE)
	}
	
	if(is.null(ode_parameters)){
		ode_parameters = createLBodeContPars(model)
	}
	
	if(!is.null(timeSignals)){
		cnolist$timeSignals = timeSignals
	}
	
	sim_function = getLBodeSimFunction(cnolist, model, adjMat,
			indices1=indices, odeParameters1=ode_parameters$parValues, time1=time, verbose1=verbose,
			transfer_function1=transfer_function, reltol1=reltol, atol1=atol, maxStepSize1=maxStepSize,
			maxNumSteps1=maxNumSteps, maxErrTestsFails1=maxErrTestsFails)
	
	sim=sim_function(cnolist,model, odeParameters=ode_parameters$parValues)
	
	sim=lapply(sim,function(x) x[,indices$signals, drop=FALSE])
	
	return(sim);
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2013 - EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
#' @export
getLBodeModelSim<-function
(
        cnolist,                model,                    ode_parameters=NULL,
        indices=NULL,            timeSignals=NULL,        time=1,
        verbose=0,                transfer_function=3,    reltol=1e-4,
        atol=1e-3,                maxStepSize=Inf,        maxNumSteps=100000,
        maxErrTestsFails=50
)
{

    # this is a temporary solution to deal with CNOlist class
    if (class(cnolist)=="CNOlist"){cnolist = compatCNOlist(cnolist)}


    adjMat=incidence2Adjacency(model);
    if(is.null(indices))indices <- indexFinder(cnolist,model,verbose=FALSE);
    if(is.null(ode_parameters))ode_parameters=createLBodeContPars(model);
    if(!is.null(timeSignals))cnolist$timeSignals=timeSignals;

    # TC aug 2013. it looks like cnolist and model parameters are useless here ?? could set to anything ??
    # smae for ode_parameters$parValues
    # need to be fixed at some point

    
    #sim_function=getLBodeSimFunction(cnolist,model,adjMat,
    #        indices, ode_parameters$parValues, time,verbose,
    #        transfer_function,reltol,atol,maxStepSize,
    #        maxNumSteps,maxErrTestsFails);

    sim_function=getLBodeSimFunction(NULL,NULL,adjMat,
            indices, NULL, time,verbose,
            transfer_function,reltol,atol,maxStepSize,
            maxNumSteps,maxErrTestsFails);

    # TC aug 2013 !!! only parValues are used in ode_parameters 
    sim_data=sim_function(cnolist,model,ode_parameters$parValues)

    sim_data=lapply(sim_data, function(x) as.matrix(x))
    
    
    return(sim_data);
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2013 - EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
#' @export
getLBodeContObjFunction<-
  function
(
  cnolist,                model,                    ode_parameters,
  indices=NULL,           time=1,                   verbose=0,
  transfer_function=3,    reltol=1e-4,            atol=1e-3,
  maxStepSize=Inf,        maxNumSteps=100000,        maxErrTestsFails=50,
  nan_fac=1, lambda_tau=0, lambda_k=0, bootstrap=F,
  SSpenalty_fac=0, SScontrolPenalty_fac=0, boot_seed=sample(1:10000,1)
)
  {
  
  if (class(cnolist)=="CNOlist"){cnolist = compatCNOlist(cnolist)}
  adjMatrix=incidence2Adjacency(model);
  
  if(is.null(indices))indices=indexFinder(cnolist,model,verbose=FALSE);
  
  sim_function=getLBodeSimFunction(cnolist,model,adjMatrix1=adjMatrix,
                                   indices1=indices, odeParameters1=ode_parameters$parValues, time1=time,verbose1=verbose,
                                   transfer_function1=transfer_function,reltol1=reltol,atol1=atol,maxStepSize1=maxStepSize,
                                   maxNumSteps1=maxNumSteps,maxErrTestsFails1=maxErrTestsFails)
  
  logic_ode_continuous_objective_function<-function
  (
    x,                            cnolist1=cnolist,        model1=model,
    adjMatrix1=adjMatrix,        indices1=indices,        ode_parameters1=ode_parameters,
    sim_function1=sim_function,        nan_fac1=nan_fac
  )
  {
    
    
    ode_parameters1$parValues[ode_parameters1$index_opt_pars]=x;
    sim=sim_function1(cnolist1,model1,ode_parameters1$parValues);
    temp=sim;
    sim=list();
    for(i in 1:length(cnolist$timeSignals)){
      sim[[i]]=temp[[i]];
    }
    
    sim<-as.vector(unlist(lapply(sim,function(x) x[,indices1$signals])));
    measured_values<-as.vector(unlist(lapply(cnolist1$valueSignals,function(x)x)));
    
    # do bootstrap if required
    if (bootstrap==T){
      res_boot<-(sim-measured_values)^2
      set.seed(boot_seed)
      res_boot<-sample(res_boot, length(res_boot), replace=T)
      error<-sum(res_boot, na.rm = T)
      # print(sample(1:5, 5, replace=T))
    }else{
      error<-sum((sim-measured_values)^2, na.rm = T)
    }
    
    NaNs_sim=which(is.na(sim));
    
    
    NApenalty<-length(NaNs_sim)*nan_fac1
    SSpenalty<-SSpenalty_fac*sum(temp[[length(cnolist$timeSignals)+1]]^2)
    SScontrolPenalty<-SScontrolPenalty_fac*sum(temp[[length(cnolist$timeSignals)+2]][1,]^2)
    L1reg_k<-lambda_k*(sum(abs(ode_parameters1$parValues[ode_parameters1$index_k])))
    L1reg_tau<-lambda_tau*(sum(abs(ode_parameters1$parValues[ode_parameters1$index_tau])))
    L1reg<-L1reg_k+L1reg_tau
    
    res=error+NApenalty+SSpenalty+SScontrolPenalty+L1reg;
    
    
    if (verbose==T){
      cat("res =", res, "\n",
          "NA penalty = ", NApenalty, "\n",
          "error =", error, "\n",
          "steady state penalty ( fac =", SSpenalty_fac, ") = ", SSpenalty, "\n",
          "control steady state penalty ( fac =", SScontrolPenalty_fac, ") = ",  SScontrolPenalty, "\n",
          "L1-reg: lambda =", lambda_tau, "- P penalty (only tau)", L1reg_tau, "\n",
          "L1-reg: lambda =", lambda_k, "- P penalty (only k)", L1reg_k, "\n",
          "L1-reg: P penalty (total)", L1reg, "\n",
          "--------\n")
    }
    
    if(is.nan(res) || is.na(res))res=1e10;
    
    return(res);
  }
  return(logic_ode_continuous_objective_function);
}

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2013 - EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
#' @export
createLBodeContPars <-function
(
        model,            LB_n=1,            LB_k=0.1,
        LB_tau=1e-2,    UB_n=5,            UB_k=0.9,
        UB_tau=10,        default_n=3,    default_k=0.5,
        default_tau=1,    LB_in=c(),        UB_in=c(),
        opt_n=TRUE,        opt_k=TRUE,        opt_tau=TRUE,
        random=FALSE
)
{
    namesSpecies=model$namesSpecies;
    #convert the graph in incidence matrix to adjacency format
    adjMat=incidence2Adjacency(model);
    #Check what species are dynamic state states and which are inputs to the system
    isState=getStates(adjMat);
    numStates=as.integer(sqrt(length(adjMat)));
    parNames=c();
    parValues=c();
    index_k=c();
    index_n=c();
    index_tau=c();
    LB=c();
    UB=c();
    index_opt_pars=c();

    #Iterate over all species
    count=0;
    for(j in 1:numStates)
    {
        #According to the odefy method, if it a state there must one Tau,
        #and one k and n for each input to that state
        #Important note: Don't change the parameter ordering scheme. These are
        #ordered according to the adjacency matrix. The names are just for guidance
        if(isState[j])
        {
            jCol=adjMat[,j];
            inputs=which(as.logical(jCol));
            numInputs=length(inputs);
            #Add k's and n's for each input
            for(i in 1:numInputs)
            {
                #Add n
                count=count+1;
                parNames[count]=paste(namesSpecies[inputs[i]],"_n_",namesSpecies[j],sep="");
                parValues[count]=default_n;
                index_n=c(index_n,count);
                LB[count]=LB_n;
                UB[count]=UB_n;
                if(opt_n)index_opt_pars=c(index_opt_pars,count);

                #add k
                count=count+1;
                parNames[count]=paste(namesSpecies[inputs[i]],"_k_",namesSpecies[j],sep="");
                parValues[count]=default_k;
                index_k=c(index_k,count);
                LB[count]=LB_k;
                UB[count]=UB_k;
                if(opt_k)index_opt_pars=c(index_opt_pars,count);
            }
            #Add tau
            count=count+1;
            parNames[count]=paste("tau_",namesSpecies[j],sep="");
            parValues[count]=default_tau;
            index_tau=c(index_tau,count);
            LB[count]=LB_tau;
            UB[count]=UB_tau;
            if(opt_tau)index_opt_pars=c(index_opt_pars,count);
        }
    }

    #You pass your predefined vectors of upper al lower bounds
    if(length(LB_in)==length(LB))LB=LB_in;
    if(length(UB_in)==length(UB))UB=UB_in;

    #Create a new uniform random solution if random==TRUE
    if(random)parValues[index_opt_pars]=LB[index_opt_pars]+((UB[index_opt_pars]-LB[index_opt_pars])*runif(length(index_opt_pars)));

    parList=list(parNames=parNames,parValues=parValues,
            index_opt_pars=index_opt_pars,index_n=index_n,
            index_k=index_k,index_tau=index_tau,LB=LB,UB=UB);

    return(parList);
}

## ----preliminaries, eval=TRUE, results='hide', include=FALSE, echo=FALSE----
options(width=70, useFancyQuotes="UTF-8", prompt=" ", continue="  ")

## ----installCNOR, eval=FALSE, pgf=TRUE, echo=TRUE-------------------
# if (!requireNamespace("BiocManager", quietly=TRUE))
#     install.packages("BiocManager")
# if (!requireNamespace("CNORode", quietly=TRUE))
# 	BiocManager::install("CNORode")

## ----installMEIGOR, eval=FALSE--------------------------------------
# # MEIGOR is deprecated in Bioc in 3.18
# #BiocManager::install("MEIGOR")
# if (!requireNamespace("MEIGOR", quietly=TRUE))
#     remotes::install_github("https://github.com/jaegea/MEIGOR/tree/RELEASE_3_18")

## ----installCNORode2, eval=TRUE-------------------------------------
library(CNORode)

## ----quickstart, eval=TRUE, results='hide'--------------------------
	library(CNORode)
	model=readSIF(system.file("extdata", "ToyModelMMB_FeedbackAnd.sif",
		package="CNORode",lib.loc = .libPaths()));
	cno_data=readMIDAS(system.file("extdata", "ToyModelMMB_FeedbackAnd.csv",
		package="CNORode",lib.loc = .libPaths()));
	cnolist=makeCNOlist(cno_data,subfield=FALSE);

## -------------------------------------------------------------------
	ode_parameters=createLBodeContPars(model, LB_n = 1, LB_k = 0.1,
		LB_tau = 0.01, UB_n = 5, UB_k = 0.9, UB_tau = 10, default_n = 3,
		default_k = 0.5, default_tau = 1, opt_n = TRUE, opt_k = TRUE,
		opt_tau = TRUE, random = FALSE)

## -------------------------------------------------------------------
	print(ode_parameters)

## ----label=plotModelSim,include=TRUE,fig=TRUE-----------------------
modelSim=plotLBodeModelSim(cnolist = cnolist, model, ode_parameters,
 	timeSignals=seq(0,2,0.5));

## ----eval=TRUE, results='hide',fig.keep = "last"--------------------
initial_pars=createLBodeContPars(model, LB_n = 1, LB_k = 0.1,
	LB_tau = 0.01, UB_n = 5, UB_k = 0.9, UB_tau = 10, random = TRUE)
#Visualize initial solution
simulatedData=plotLBodeFitness(cnolist, model,initial_pars)

## ----eval=TRUE, results='hide',fig.keep = "last"--------------------
paramsGA = defaultParametersGA()
paramsGA$maxStepSize = 1
paramsGA$popSize = 50
paramsGA$iter = 100
paramsGA$transfer_function = 2
paramsGA$monitor = FALSE
opt_pars=parEstimationLBode(cnolist,model,ode_parameters=initial_pars,
	paramsGA=paramsGA)

## ----eval=TRUE, results='hide',fig.keep = "last"--------------------
#Visualize fitted solution
simulatedData=plotLBodeFitness(cnolist, model,ode_parameters=opt_pars)

## ----eval=FALSE, results='hide'-------------------------------------
# 
# requireNamespace("MEIGOR")
# 
# 
# initial_pars=createLBodeContPars(model,
# 	LB_n = 1, LB_k = 0.1, LB_tau = 0.01, UB_n = 5,
# 	UB_k = 0.9, UB_tau = 10, random = TRUE)
# #Visualize initial solution
# 
# fit_result_ess =
# 	parEstimationLBodeSSm(cnolist = cnolist,
# 						  model = model,
# 						  ode_parameters = initial_pars,
# 						  maxeval = 1e5,
# 						  maxtime = 20,
# 						  local_solver = "DHC",
# 						  transfer_function = 3
# 	)
# #Visualize fitted solution
# # simulatedData=plotLBodeFitness(cnolist, model,ode_parameters=fit_result_ess)

## ----label=plotInit,include=TRUE,fig=TRUE---------------------------
	simulatedData=plotLBodeFitness(cnolist, model,
								   initial_pars,
								   transfer_function = 3)

## ----label=plotFinalFit_fit,eval = FALSE----------------------------
# 	simulatedData=plotLBodeFitness(cnolist, model,
# 								   ode_parameters=fit_result_ess,
# 								   transfer_function = 3)

## ----eval=FALSE-----------------------------------------------------
# library(MEIGOR)
# f_hepato<-getLBodeContObjFunction(cnolist, model, initial_pars, indices=NULL,
#  time = 1, verbose = 0, transfer_function = 2, reltol = 1e-05, atol = 1e-03,
# maxStepSize = Inf, maxNumSteps = 1e4, maxErrTestsFails = 50, nan_fac = 1)
# n_pars=length(initial_pars$LB);
# 
# problem<-list(f=f_hepato, x_L=initial_pars$LB[initial_pars$index_opt_pars],
# 	x_U=initial_pars$UB[initial_pars$index_opt_pars]);
# 
# #Source a function containing the options used in the CeSSR publication
#  source(system.file("benchmarks","get_paper_settings.R",package="MEIGOR"))
# #Set max time as 20 seconds per iteration
# opts<-get_paper_settings(20);
# Results<-CeSSR(problem,opts,Inf,Inf,3,TRUE,global_save_list=c('cnolist','model',
# 'initial_pars'))

## ----eval=FALSE-----------------------------------------------------
# library(CellNOptR)
# library(CNORode)
# library(MEIGOR)
# 
# # MacNamara et al. 2012 case study:
# data(PKN_ToyPB, package="CNORode")
# data(CNOlist_ToyPB, package="CNORode")
# 
# # original and preprocessed network
# plotModel(pknmodel, cnodata)
# model = preprocessing(data = cnodata, model = pknmodel,
#                       compression = T, expansion = T)
# plotModel(model, cnodata)
# plotCNOlist(CNOlist = cnodata)
# 
# # set initial parameters
# ode_parameters=createLBodeContPars(model, LB_n = 1, LB_k = 0,
#                                    LB_tau = 0, UB_n = 4, UB_k = 1,
#                                    UB_tau = 1, default_n = 3, default_k = 0.5,
#                                    default_tau = 0.01, opt_n = FALSE, opt_k = TRUE,
#                                    opt_tau = TRUE, random = TRUE)
# 
# ## Parameter Optimization
# # essm
# paramsSSm=defaultParametersSSm()
# paramsSSm$local_solver = "DHC"
# paramsSSm$maxtime = 600;
# paramsSSm$maxeval = Inf;
# paramsSSm$atol=1e-6;
# paramsSSm$reltol=1e-6;
# paramsSSm$nan_fac=0;
# paramsSSm$dim_refset=30;
# paramsSSm$n_diverse=1000;
# paramsSSm$maxStepSize=Inf;
# paramsSSm$maxNumSteps=10000;
# transferFun=4;
# paramsSSm$transfer_function = transferFun;
# 
# paramsSSm$lambda_tau=0
# paramsSSm$lambda_k=0
# paramsSSm$bootstrap=F
# paramsSSm$SSpenalty_fac=0
# paramsSSm$SScontrolPenalty_fac=0
# 
# # run the optimisation algorithm
# opt_pars=parEstimationLBode(cnodata,model, method="essm",
#                             ode_parameters=ode_parameters, paramsSSm=paramsSSm)
# plotLBodeFitness(cnolist = cnodata, model = model,
#                  ode_parameters = opt_pars, transfer_function = 4)
# 
# # 10-fold crossvalidation using T1 data
# # We use only T1 data for crossvalidation, because data
# # in the T0 matrix is not independent.
# # All rows of data in T0 describes the basal condition.
# 
# # Crossvalidation produce some text in the command window:
# library(doParallel)
# registerDoParallel(cores=3)
# R=crossvalidateODE(CNOlist = cnodata, model = model,
#                    type="datapoint", nfolds=3, parallel = TRUE,
#                    ode_parameters = ode_parameters, paramsSSm = paramsSSm)

#
#  This file is part of the CNO software
#
#  Copyright (c) 2011-2013 - EBI
#
#  File author(s): CNO developers (cno-dev@ebi.ac.uk)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  CNO website: http://www.cellnopt.org
#
##############################################################################
#' @export
plotLBodeFitness <-function
(
        cnolist,                    model,                        ode_parameters=NULL,
        indices=NULL,                adjMatrix=NULL,                 time=1,
        verbose=0,                     transfer_function=3,        reltol=1e-4,
        atol=1e-3,                    maxStepSize=Inf,             maxNumSteps=100000,
        maxErrTestsFails=50,        plot_index_signals=NULL,    plot_index_experiments=NULL,
        plot_index_cues=NULL,         colormap="heat",
        plotParams=list(margin=0.1, width=15, height=12,
                  cmap_scale=1, cex=1.6, ymin=NULL)
  

)
{

    if (class(cnolist)=="CNOlist"){ cnolist = compatCNOlist(cnolist)}

    if(is.null(plot_index_experiments))plot_index_experiments=1:dim(cnolist$valueCues)[1];
    if(is.null(plot_index_cues))plot_index_cues=1:dim(cnolist$valueCues)[2];
    if(is.null(plot_index_signals))plot_index_signals=1:dim(cnolist$valueSignals[[1]])[2];
    if(is.null(indices))indices=indexFinder(cnolist,model);
    if(is.null(adjMatrix))adjMatrix=incidence2Adjacency(model);
    if(is.null(ode_parameters))ode_parameters=createLBodeContPars(model);
    timeSignals=NULL;

    sim_data=getLBodeDataSim(cnolist,model,
            ode_parameters,indices,timeSignals,time,verbose,
            transfer_function,reltol,atol,maxStepSize,maxNumSteps,
            maxErrTestsFails);
            
            
            temp=list();
            
            for(i in 1:length(cnolist$timeSignals)){
                
                temp[[i]]=sim_data[[i]];
            }
            
            sim_data=temp;

    times=cnolist$timeSignals;

    sim_data=lapply(sim_data,function(x)x[plot_index_experiments,plot_index_signals]);
    
   
    
    sim_data=temp;
    
    
    expResults=lapply(cnolist$valueSignals,function(x)x[plot_index_experiments,plot_index_signals]);

    sim_data=lapply(sim_data,function(x)as.matrix(x));
    expResults=lapply(expResults,function(x)as.matrix(x));

    if(dim(cnolist$valueSignals[[1]])[1]!=dim(sim_data[[1]])[1]){
        sim_data=lapply(sim_data,function(x) t(as.matrix(x)));
    }

    namesSignals=cnolist$namesSignals[plot_index_signals];
    namesCues=c(cnolist$namesStimuli,cnolist$namesInhibitors);

    valueCues=cbind(cnolist$valueStimuli,cnolist$valueInhibitors);
    valueCues=as.matrix(valueCues);
    valueCues[which(valueCues>0)]=1;
    valueCues=valueCues[plot_index_experiments,plot_index_cues];
    names(valueCues)=namesCues[plot_index_cues];

    if (colormap=="green"){
        plotOptimResults(simResults=sim_data,expResults=expResults,
        times=times,namesCues=namesCues,namesSignals=namesSignals,
        valueCues=valueCues);
    } else{
        plotOptimResultsPan(sim_data, yInterpol=NULL, xCoords=NULL,
             CNOlist=CNOlist(cnolist), formalism="ode", pdf=FALSE,
             plotParams=plotParams,pdfFileName="", tPt=NULL)
    }

  return(sim_data);
}

---
title:  'Training Signalling Pathway Maps to Biochemical Data with Logic-Based Ordinary Differential Equations'
author: David Henriques, Thomas Cokelaer, Enio Gjerga, Attila Gabor
date: "`r format(Sys.time(), '%d %B %Y')`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Vignette Title}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r preliminaries, eval=TRUE, results='hide', include=FALSE, echo=FALSE}
options(width=70, useFancyQuotes="UTF-8", prompt=" ", continue="  ")
```

## Introduction

Mathematical models are used to understand protein signalling networks so as to
provide an integrative view of pharmacological and toxicological processes at
molecular level. *CellNOptR*~@CellNOptR is an existing
package (see
(http://bioconductor.org/packages/release/bioc/html/CellNOptR.html)) that
provides functionalities to combine
prior knowledge network (about protein signalling networks) and perturbation
data to infer functional characteristics (of the signalling network).
While *CellNOptR* has demonstrated its ability to infer new functional
characteristics, it is based on a boolean formalism where protein species are
characterised as being fully active or inactive. In contrast, logic-based ordinary
differential equations allow a quantitive description of a given Boolean model.

The method used here was first published by Wittmann et al. @wittman by
the name of *odefy*.
For a detailed description of the methodology the user is adressed to @wittman and for
a published application example to @wittmanJS.

This package implements the Odefy method and focus mainly extending the *CellNOptR*
capabilities in order to simulate and calibrate logic-based ordinary differential
equation model. We provide direct and easy to use interface to optimization methods
available in R such as *eSSR* @eSSR1 (enhanced Scatter Search Metaheuristic for R) and an R genetic
algorithm implementation by the name of *genalg* in order to perform parameter estimation.
Additionally we were specially careful in tackling the main computanional bottlenecks
by implementing CNORode simulation engine in the C language using the
*CVODES* library @cvodes.

This brief tutorial shows how to use *CNORode* using as a starting point a Boolean model
and a dataset consisting in a time-series of several proteins.

## Installation

*CNORode* depends on *CellNOptR* and *genalg*, which are 2 bioconductor
packages. Therefore, in order to install *CNORode*, open a R
session and type:

```{r installCNOR, eval=FALSE, pgf=TRUE, echo=TRUE}
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
if (!requireNamespace("CNORode", quietly=TRUE))
	BiocManager::install("CNORode")
```

It may take a few minutes to install all dependencies if you start from scratch
(i.e, none of the R packages are installed on your system). Note also that under
Linux system, some of these packages necessitate the R-devel package to be
installed (e.g., under Fedora type *sudo yum install R-devel*).

Additionally, for parameter estimation we recommend the use of *eSSR*. This
algorithm  is part of the MEIGOR toolbox which was available on BioConductor (before release 3.19, https://www.bioconductor.org/packages/release/bioc/html/MEIGOR.html.  ). Currently, it can be 
 can be downloaded from GitHub: https://github.com/jaegea/MEIGOR/tree/RELEASE_3_18. 

MEIGOR can be installed by typing

```{r installMEIGOR, eval=FALSE}
# MEIGOR is deprecated in Bioc in 3.18
#BiocManager::install("MEIGOR")
if (!requireNamespace("MEIGOR", quietly=TRUE))
    remotes::install_github("https://github.com/jaegea/MEIGOR/tree/RELEASE_3_18")
```

Finally, once *CNORode* is installed you can load it by typing:

```{r installCNORode2, eval=TRUE}
library(CNORode)
```

## Quick Start

In this section, we provide a quick example on how to use *CNORode* to
find the set of continuous parameters which minimize the squared difference
between a model simulation and the experimental data.

Since here we will not be modifying the model structure as opposed to *CellNOptR*
we will use a model that already contains AND type gates. Such model can be for instance
the result of calibrating a *prior knowledge network*(PKN) with *CellNOptR*.
Please note that a PKN can also be used as Boolean model which will contain only OR type gates.

Detailed information about the model used here (ToyModelMMB\_FeedbackAnd) and additional models can be found at:
(https://saezlab.github.io/CellNOptR/5_Models%20and%20Documentation/)


\begin{center}
\begin{figure}[ht]
\includegraphics[height=7cm, width=7cm]{ToyModelMMB_FeedbackAnd}
\includegraphics[height=7cm, width=7cm]{data_ToyModelMMB_FeddbackAnd}
\caption{The used model(left panel). A plot from the data, resulting from
the *plotCNOlist* function (right panel).}
\end{figure}
\end{center}

The example used here is shipped with the CNORode. In order to load the data and model you
should type the following commands:

show data and model loading
```{r quickstart, eval=TRUE, results='hide'}
	library(CNORode)
	model=readSIF(system.file("extdata", "ToyModelMMB_FeedbackAnd.sif",
		package="CNORode",lib.loc = .libPaths()));
	cno_data=readMIDAS(system.file("extdata", "ToyModelMMB_FeedbackAnd.csv",
		package="CNORode",lib.loc = .libPaths()));
	cnolist=makeCNOlist(cno_data,subfield=FALSE);
```

The structure from the CNOlist and the Model object is exactly the same as used in the
*CellNOptR* and therefore for a detailed explanation about these structure we direct
the reader to the *CellNOptR* manual.

In order to simulate the model and perform parameter estimation we first need to create
a list with the ODE parameters associated with each dynamic state as described in @wittman.
Each dynamic state will have a \(\tau\) parameter, as many \(n\) and \(k\) parameters as inputs.
Although the default is to use the normalized Hill function it also possible to use the
standard Hill or even not to use any transfer function at all. To illustrate the shape of the equations
associated to each dynamic state and the meaning of each parameter, let us show the differential
of *Mek*:


\[ \dot{Mek}=\Bigg[  \Bigg( 1 - \frac{Akt^{n_1}/({k_1}^{n_1} + Akt^{n_1})}{1/({k_1}^{n_1} + 1)}\Bigg)
	\cdot \Bigg(\frac{Raf^{n_2}/({k_2}^{n_2} + Raf^{n_2})}{1/({k_2}^{n_2} + 1)}\Bigg) - Mek \Bigg] \cdotp \tau_{Mek}
\]

To create a list of ODE parameters we will typically use the *createLBodeContPars* function:

```{r }
	ode_parameters=createLBodeContPars(model, LB_n = 1, LB_k = 0.1,
		LB_tau = 0.01, UB_n = 5, UB_k = 0.9, UB_tau = 10, default_n = 3,
		default_k = 0.5, default_tau = 1, opt_n = TRUE, opt_k = TRUE,
		opt_tau = TRUE, random = FALSE)
```

This function creates a general structure where the ODE parameters are ordered according
to the model. Some tweaks have been added in order to ease tasks we have found
to be common, nevertheless you can edit several attributes manually. If you print the
\(ode\_parameters\) list you will see the following attributes.


```{r }
	print(ode_parameters)
```
\clearpage

Typically before running an optimization run you will want to choose which type of parameters
you want to optimize. The field \emph{index\_opt\_pars} defines which parameters are meant to be optimized.
In the *createLBodeContPars*, if you choose \(opt\_tau \) as \(TRUE\)  all \( \tau \) parameters will be
added to the index\_opt\_pars array, the same idea is valid for \( n \) and \( k \) parameters.

It is also possible to choose default values for lower and upper bounds for the parameters of a given type,
e.g. \( \tau \) ( \emph{LB\_tau} and \emph{UB\_tau}), as well as a default initial value for such parameters.

Once we have the ODE parameters structure we are ready to run a simulation or optimization process.
To run a simulation we can use the *getLBodeModel* or *getLBodeDataSim*, depending on
if we want to simulate only the signals present in the CNOlist object or all the species in the model. Additionally
*plotLBodeDataSim* or *plotLBodeModelSim* will also return the values of a model simulation while
plotting the same values. In figure \@ref(fig:plotModelSimFig), we use plotLBodeModelSim to plot all the experiments
sampled in 5 different instants (*timeSignals*).


```{r label=plotModelSim,include=TRUE,fig=TRUE}
modelSim=plotLBodeModelSim(cnolist = cnolist, model, ode_parameters,
 	timeSignals=seq(0,2,0.5));
```

\clearpage

As previously mentioned, we provide two optimization algorithms that allow parameter estimation
Both of these algorithms have specific parameters that can be tunned on each specific problem
(please check CNORode manual for detailed information). For instance, in order to run the
genetic algorithm for 10 iterations and a population of size of 10, we can use the following code:

```{r eval=TRUE, results='hide',fig.keep = "last"}
initial_pars=createLBodeContPars(model, LB_n = 1, LB_k = 0.1,
	LB_tau = 0.01, UB_n = 5, UB_k = 0.9, UB_tau = 10, random = TRUE)
#Visualize initial solution
simulatedData=plotLBodeFitness(cnolist, model,initial_pars)
```


```{r eval=TRUE, results='hide',fig.keep = "last"}
paramsGA = defaultParametersGA()
paramsGA$maxStepSize = 1
paramsGA$popSize = 50
paramsGA$iter = 100
paramsGA$transfer_function = 2
paramsGA$monitor = FALSE
opt_pars=parEstimationLBode(cnolist,model,ode_parameters=initial_pars,
	paramsGA=paramsGA)
```


```{r eval=TRUE, results='hide',fig.keep = "last"}
#Visualize fitted solution
simulatedData=plotLBodeFitness(cnolist, model,ode_parameters=opt_pars)
```

Model optimization using eSS (not evaluated):

```{r eval=FALSE, results='hide'}

requireNamespace("MEIGOR")


initial_pars=createLBodeContPars(model,
	LB_n = 1, LB_k = 0.1, LB_tau = 0.01, UB_n = 5,
	UB_k = 0.9, UB_tau = 10, random = TRUE)
#Visualize initial solution

fit_result_ess = 
	parEstimationLBodeSSm(cnolist = cnolist,
						  model = model,
						  ode_parameters = initial_pars,
						  maxeval = 1e5,
						  maxtime = 20,
						  local_solver = "DHC",
						  transfer_function = 3
	)
#Visualize fitted solution
# simulatedData=plotLBodeFitness(cnolist, model,ode_parameters=fit_result_ess)
```





```{r label=plotInit,include=TRUE,fig=TRUE}
	simulatedData=plotLBodeFitness(cnolist, model,
								   initial_pars,
								   transfer_function = 3)
```

The initial solution before optimization. Each row corresponds to an experiment with a particular combination of stimuli and inhibitors.
The columns correspond to the measured values (triangles) and the simulated values (dashed blue lines) from a given signal. The background color
gives an indication of squared difference where red means high error and white low error.



```{r label=plotFinalFit_fit,eval = FALSE}
	simulatedData=plotLBodeFitness(cnolist, model,
								   ode_parameters=fit_result_ess,
								   transfer_function = 3)
```



\clearpage

In addition to eSSR and genalg its is fairly easy to use any other continuous optimization algorithm.
In the following example we show how to generate and use an the objective function in order to use
 it with a variant of eSSR(part of MEIGOR package) that uses multiple cpus:

```{r eval=FALSE}
library(MEIGOR)
f_hepato<-getLBodeContObjFunction(cnolist, model, initial_pars, indices=NULL,
 time = 1, verbose = 0, transfer_function = 2, reltol = 1e-05, atol = 1e-03,
maxStepSize = Inf, maxNumSteps = 1e4, maxErrTestsFails = 50, nan_fac = 1)
n_pars=length(initial_pars$LB);

problem<-list(f=f_hepato, x_L=initial_pars$LB[initial_pars$index_opt_pars],
	x_U=initial_pars$UB[initial_pars$index_opt_pars]);

#Source a function containing the options used in the CeSSR publication
 source(system.file("benchmarks","get_paper_settings.R",package="MEIGOR"))
#Set max time as 20 seconds per iteration
opts<-get_paper_settings(20);
Results<-CeSSR(problem,opts,Inf,Inf,3,TRUE,global_save_list=c('cnolist','model',
'initial_pars'))
```

## Crossvalidation

CNORode offers the possibility to perform a k-fold cross-validation for logic-ode models in order to assess the predictive performance of our models. In k-iterations a  fraction of the data is eliminated from the CNOlist. The model is trained on the  remaining data and then the model predicts the held-out data. Then the prediction accuracy is reported for each iteration. Three different re-sampling strategies about how we can split the training and the test set: *1)*Re-sampling of the data-points, *2)*Re-sampling of the experimental conditions and *3)*Resampling of the observable nodes.

In the example below, we show an example about how we can apply the cross-validation analysis over a small toy case-study from Macnamara et al. 2012.

```{r eval=FALSE}
library(CellNOptR)
library(CNORode)
library(MEIGOR)

# MacNamara et al. 2012 case study:
data(PKN_ToyPB, package="CNORode")
data(CNOlist_ToyPB, package="CNORode")

# original and preprocessed network 
plotModel(pknmodel, cnodata)
model = preprocessing(data = cnodata, model = pknmodel,
                      compression = T, expansion = T)
plotModel(model, cnodata)
plotCNOlist(CNOlist = cnodata)

# set initial parameters 
ode_parameters=createLBodeContPars(model, LB_n = 1, LB_k = 0,
                                   LB_tau = 0, UB_n = 4, UB_k = 1, 
                                   UB_tau = 1, default_n = 3, default_k = 0.5, 
                                   default_tau = 0.01, opt_n = FALSE, opt_k = TRUE,
                                   opt_tau = TRUE, random = TRUE)

## Parameter Optimization
# essm
paramsSSm=defaultParametersSSm()
paramsSSm$local_solver = "DHC"
paramsSSm$maxtime = 600;
paramsSSm$maxeval = Inf;
paramsSSm$atol=1e-6;
paramsSSm$reltol=1e-6;
paramsSSm$nan_fac=0;
paramsSSm$dim_refset=30;
paramsSSm$n_diverse=1000;
paramsSSm$maxStepSize=Inf;
paramsSSm$maxNumSteps=10000;
transferFun=4;
paramsSSm$transfer_function = transferFun;

paramsSSm$lambda_tau=0
paramsSSm$lambda_k=0
paramsSSm$bootstrap=F
paramsSSm$SSpenalty_fac=0
paramsSSm$SScontrolPenalty_fac=0

# run the optimisation algorithm
opt_pars=parEstimationLBode(cnodata,model, method="essm", 
                            ode_parameters=ode_parameters, paramsSSm=paramsSSm)
plotLBodeFitness(cnolist = cnodata, model = model, 
                 ode_parameters = opt_pars, transfer_function = 4)

# 10-fold crossvalidation using T1 data
# We use only T1 data for crossvalidation, because data 
# in the T0 matrix is not independent.
# All rows of data in T0 describes the basal condition.

# Crossvalidation produce some text in the command window:  
library(doParallel)
registerDoParallel(cores=3)
R=crossvalidateODE(CNOlist = cnodata, model = model, 
                   type="datapoint", nfolds=3, parallel = TRUE, 
                   ode_parameters = ode_parameters, paramsSSm = paramsSSm)
```

For more, please information about the *crossvalidateODE* function, please check its documentation.

\begin{thebibliography}{}

\bibitem{CellNOptR}
C.~Terfve.
\newblock CellNOptR: R version of CellNOpt, boolean features only.
\newblock {R package version 1.2.0, (2012)
http://www.bioconductor.org/packages/release/bioc/html/CellNOptR.html}

\bibitem{alexopoulos_networks_2010}
L.G.~Alexopoulos, J.~Saez-Rodriguez, B.D.~Cosgrove, D.A.~Lauffenburger,
P.K~Sorger.: Networks inferred from biochemical data reveal profound
  differences in toll-like receptor and inflammatory signaling between normal
  and transformed hepatocytes.
\newblock Molecular \& Cellular Proteomics: {MCP} **9**(9), 1849--1865
  (2010).

\bibitem{MMB}
M.K.~Morris, I.~Melas, J.~Saez-Rodriguez.
\newblock Construction of cell type-specific logic models of signalling networks using CellNetOptimizer.
\newblock {\em Methods in Molecular Biology: Computational Toxicology}, Ed. B. Reisfeld and A. Mayeno, Humana Press.

\bibitem{fuzzy2011}
M.K.~Morris, J.~Saez-Rodriguez, D.C.~Clarke, P.K.~Sorger, D.A.~Lauffenburger.
\newblock Training Signaling Pathway Maps to Biochemical Data with Constrain
ed Fuzzy Logic: Quantitative Analysis of Liver Cell Responses to Inflammatory
Stimuli.
\newblock{\em PLoS Comput Biol.} 7(3) (2011) : e1001099.

\bibitem{julio2009}
J.~Saez-Rodriguez, L.~Alexopoulos, J.~Epperlein, R.~Samaga, D.~Lauffenburger, S.~Klamt and P.K.~Sorger.
\newblock Discrete logic modelling as a means to link protein signalling networks with functional analysis of mammalian signal transduction.
\newblock {\em Molecular Systems Biology}, 5:331, 2009.

\bibitem{wittmanJS}
Dominik Wittmann, Jan Krumsiek, Julio S. Rodriguez, Douglas
Lauffenburger, Steffen Klamt, and Fabian Theis.
\newblock Transforming boolean models to continuous models: methodology and application to t-cell
receptor signaling.
\newblock BMC Systems Biology, 3(1):98+, September 2009.

\bibitem{eSSR1}
Egea, J.A., Maria, R., Banga, J.R. (2010)
\newblock An evolutionary method for complex-process optimization.
\newblock Computers \& Operations Research 37(2):315\-324.

\bibitem{eSSR2}
Egea, J.A., Balsa-Canto, E., Garcia, M.S.G., Banga, J.R. (2009)
\newblock Dynamic optimization of nonlinear processes with an enhanced scatter search
method.
\newblock Industrial \& Engineering Chemistry Research 49(9): 4388\-4401.

\bibitem{wittman}
Jan Krumsiek, Sebastian Polsterl, Dominik Wittmann, and Fabian Theis.
\newblock Odefy - from discrete to continuous models.
\newblock BMC Bioinformatics, 11(1):233+, 2010.

\bibitem{cvodes}
R. Serban and A. C. Hindmarsh
\newblock"CVODES: the Sensitivity\-Enabled ODE Solver in SUNDIALS,"
\newblock Proceedings of IDETC/CIE 2005, Sept. 2005, Long Beach, CA. Also available as LLNL
technical report UCRL\-JP\-200039.

\bibitem{macnamara_2012}
A. MacNamara, C. Terfve, D. Henriques, B.P. Bernabe, and J. Saez-Rodriguez.
\newblock State-time spectrum of signal transduction logic models.
\newblock Phys Biol., 9(4):045003, 2012.

\end{thebibliography}

\name{plotLBodeFitness}

\alias{plotLBodeFitness}
%- Also NEED an '\alias' for EACH other topic documented here.

\title{
	Plot data against simulated values. 
}

\description{
	Plots the simulated values with the logic-based ODE against the the data contained
	contained the data contained in the cnolist. The data values are represented with
	a black line and the simulated values with a blue line. Additionally this functions
	returns the the simulated values.
}

\usage{
	plotLBodeFitness(cnolist, model, ode_parameters = NULL, indices = NULL,
 		adjMatrix = NULL, time = 1, verbose = 0, transfer_function = 3, reltol = 1e-04,
		atol = 0.001, maxStepSize = Inf, maxNumSteps = 1e+05, maxErrTestsFails = 50,
   		plot_index_signals = NULL, plot_index_experiments = NULL,
plot_index_cues = NULL, colormap="heat", plotParams=list(margin=0.1, width=15, height=12,
                  cmap_scale=1, cex=1.6, ymin=NULL)
  

)
}

%- maybe also 'usage' for other objects documented here.
\arguments{
 	\item{cnolist}{
		A list containing the experimental design and data.
	}
	\item{model}{
		The logic model to be simulated.
	}
	\item{ode_parameters}{
		A list with the ODEs parameter information. Obtained with \code{\link{createLBodeContPars}}.
	}
 	\item{indices}{	
		Indices to map data in the model. Obtained with indexFinder function from CellNOptR. 
	}
	\item{adjMatrix}{
		Model representation in the form of an adjacency matrix. When not provided
		will be automatically computed based in the model.
	}
  	\item{time}{
		An integer with the index of the time point to start the simulation. Default is 1.
	}
  	\item{verbose}{
		A logical value that triggers a set of comments.
	}
	\item{transfer_function}{
		The type of used transfer. Use 1 for no transfer function, 2 for Hill function and
		3 for normalized Hill function.
	}
	\item{reltol}{
		Relative Tolerance for numerical integration.
	}
	\item{atol}{
		Absolute tolerance for numerical integration.
	}
	\item{maxStepSize}{
		The maximum step size allowed to ODE solver.
	}
	\item{maxNumSteps}{
		The maximum number of internal steps between two points being sampled before
		the solver fails.
	}
	\item{maxErrTestsFails}{
		Specifies the maximum number of error test failures permitted in attempting
	 	one step.
	}
  	\item{plot_index_signals}{
		In case you only want to plot some signals, provide an integer vector with  the indexes.
	}
	\item{plot_index_experiments}{
		In case you only want to plot some experiments, provide an integer vector with  the indexes.
	}
  	\item{plot_index_cues}{
		In case you only want to plot some cues, provide an integer vector with  the indexes.
	}
    \item{colormap}{
    Uses the same colormap as in CellNOptR by default. If set to "green", it uses the deprecated colormap.
    }

    \item{plotParams}{
    additional parameters to refine the ploggin. See plotOptimResultsPan
function in CellNOptR for more details.
}
}

\details{
	Check \code{\link{CellNOptR}} for details about the cnolist and the model format.
	For more details in the configuration of the ODE solver check the CVODES manual. 
}

\value{
	Returns a list with simulated data that has the same structure as the cnolist$valueSignals.
	One matrix for each time-point.
}

\author{
	David Henriques, Thomas Cokelaer
}

\seealso{
	\code{\link{CellNOptR}}
	\code{\link{createLBodeContPars}}
}

\examples{
	library(CNORode)
	data("ToyCNOlist",package="CNORode");
	data("ToyModel",package="CNORode");
	data("ToyIndices",package="CNORode");
	ode_parameters=createLBodeContPars(model,random=TRUE);
	dataSimulation=plotLBodeFitness(cnolistCNORodeExample, model,indices=indices);
}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.

\name{CNORode}

\alias{CNORode}

\docType{package}

\title{
	Logic based ODE extension for CellNOptR
}
\description{
	This package is used for the simulation and fitting of
	logic based ODE models based on the Odefy approach.
}

\details{
	\tabular{ll}{
		Package: \tab CNORode\cr
		Type: \tab Package\cr
		Version: \tab 1.2.0\cr
		Date: \tab 2012-03-14\cr
		License: \tab GPL-3\cr
		LazyLoad: \tab yes\cr
	}
}

\author{
	David Henriques, Thomas Cokelaer
	Maintainer: David Henriques <dhenriques@ebi.ac.uk>
}

\references{
	Dominik Wittmann, Jan Krumsiek, Julio S. Rodriguez, Douglas
	Lauffenburger, Steffen Klamt, and Fabian Theis. Transforming boolean
	models to continuous models: methodology and application to t-cell
	receptor signaling. BMC Systems Biology, 3(1):98+, September 2009.

	Egea, J.A., Maria, R., Banga, J.R. (2010) An evolutionary method for
	complex-process optimization. Computers & Operations Research 37(2):
	315-324.
 
	Egea, J.A., Balsa-Canto, E., Garcia, M.S.G., Banga, J.R. (2009) Dynamic
	optimization of nonlinear processes with an enhanced scatter search
	method. Industrial & Engineering Chemistry Research 49(9): 4388-4401. 

	Jan Krumsiek, Sebastian Polsterl, Dominik Wittmann, and Fabian
	Theis. Odefy - from discrete to continuous models. BMC
	Bioinformatics, 11(1):233+, 2010.

	R. Serban and A. C. Hindmarsh, "CVODES: the Sensitivity-Enabled ODE Solver in SUNDIALS,"
	Proceedings of IDETC/CIE 2005, Sept. 2005, Long Beach, CA. Also available as LLNL
	technical report UCRL-JP-200039.     

    C. Terfve, T. Cokelaer, A. MacNamara, D. Henriques, E. Goncalves, MK.
    Morris, M. van Iersel, DA Lauffenburger, J. Saez-Rodriguez.
    CellNOptR: a flexible toolkit to train protein signaling networks to data using
    multiple logic formalisms. BMC Systems Biology, 2012, 6:133:
}
\keyword{CNORode}

\seealso{
	\code{\link{CellNOptR}},
	\code{\link{parEstimationLBode}},
	\code{\link{getLBodeModelSim}},
	\code{\link{parEstimationLBode}} 
	\code{\link{plotLBodeFitness}}.
}

\name{parEstimationLBode}

\alias{parEstimationLBode}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
	Perform parameter estimation using a genetic algorithm (package genalg) or ssm
	(if package essm available).
}

\description{
	This function is an alias to the parEstimationLBode variants
	(\code{\link{parEstimationLBodeGA}} and \code{\link{parEstimationLBodeSSm}})
}

\usage{
	parEstimationLBode(cnolist, model, method="ga", ode_parameters = NULL, indices = NULL,
	paramsGA=NULL, paramsSSm=NULL)
}

%- maybe also 'usage' for other objects documented here.
\arguments{
 	\item{cnolist}{
		A list containing the experimental design and data.
	}
	\item{model}{
		The logic model to be simulated.
	}
	\item{method}{
		Only "ga" or "essm" arguments are accepted.
	}
 	\item{ode_parameters}{
		A list with the ODEs parameter information. Obtained with \code{\link{createLBodeContPars}}.
	}
	\item{indices}{
		Indices to map data in the model. Obtained with indexFinder function from CellNOptR. 
	}
	\item{paramsGA}{
		 A list of GA parameters. default is the list returned by \code{defaultParametersGA}.
	} 
	\item{paramsSSm}{
		 A list of SSm parameters. default is the list returned by\code{defaultParametersSSm}.
	} 
}

\value{
	\item{LB_n}{
		A numeric value to be used as lower bound for all parameters of type n.
	}
	\item{LB_k}{
		A numeric value to be used as lower bound for all parameters of type k.
	}

	\item{LB_tau}{
		A numeric value to be used as lower bound for all parameters of type tau.
	}
	\item{UB_n}{
		A numeric value to be used as upper bound for all parameters of type n.
	}
  	\item{UB_k}{
		A numeric value to be used as upper bound for all parameters of type k.
	}
	\item{UB_tau}{
		A numeric value to be used as upper bound for all parameters of type tau. 
	}
	\item{default_n}{
		The default parameter to be used for every parameter of type n. 
	}
	\item{default_k}{
		The default parameter to be used for every parameter of type k. 
	}
	\item{default_tau}{
		The default parameter to be used for every parameter of type tau. 
	}
	\item{LB_in}{
		An array with the the same length as ode_parameters$parValues with lower bounds
		for each specific parameter. 
	}
	\item{UB_in}{
		An array with the the same length as ode_parameters$parValues with upper bounds
		for each specific parameter.
	}
	\item{opt_n}{
		Add all parameter n to the index of parameters to be fitted.
	}
	\item{opt_k}{
		Add all parameter k to the index of parameters to be fitted.
	}
	\item{opt_tau}{
		Add all parameter tau to the index of parameters to be fitted.
	}
	\item{random}{
		A logical value that determines that a random solution is for the parameters 
		to be optimized.
	}
	\item{res}{A list containing the information provided by the solver.}
}

\author{
	David Henriques, Thomas Cokelaer
}

%% ~Make other sections like Warning with \section{Warning }{....} ~
\seealso{
	\code{\link{CellNOptR}}
	\code{\link{createLBodeContPars}}
	\code{\link{rbga}}
}

\examples{
data("ToyCNOlist",package="CNORode");
data("ToyModel",package="CNORode");
data("ToyIndices",package="CNORode");
	
ode_parameters=createLBodeContPars(model,random=TRUE);
#Visualize initial solution
simulatedData=plotLBodeFitness(cnolistCNORodeExample, model,ode_parameters,indices=indices)
paramsGA = defaultParametersGA()
paramsGA$maxStepSize = 1
paramsGA$popSize = 10
paramsGA$iter = 10
paramsGA$transfer_function = 2
	
ode_parameters=parEstimationLBode(cnolistCNORodeExample,model,ode_parameters=ode_parameters,
	paramsGA=paramsGA)
#Visualize fitted solution
simulatedData=plotLBodeFitness(cnolistCNORodeExample, model,ode_parameters,indices=indices)
}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{genetic}
\keyword{algorithm}% __ONLY ONE__ keyword per line
\keyword{CellNOptR}
\keyword{logic}
\keyword{model}

\name{getLBodeMINLPObjFunction}

\alias{getLBodeMINLPObjFunction}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{
	Get the objective function to evaluate the fitness of a given
	model structure and set of parameters.
}

\description{
	This function configures returns the objective function that can be used to evaluate
	the fitness of a logic based ODE model using a particular set of parameters and model
	structure. This function can be particular useful if you are planing to couple a
	mixed integer nonlinear programming optimization solver. The returned value of the 
	objective function corresponds to the mean squared value.
}

\usage{
	getLBodeMINLPObjFunction(cnolist, model, ode_parameters, indices=NULL, time = 1, 
	verbose = 0, transfer_function = 3, reltol = 1e-04, atol = 0.001, maxStepSize = Inf, 
	maxNumSteps = 1e+05, maxErrTestsFails = 50, nan_fac = 1)
}

%- maybe also 'usage' for other objects documented here.
\arguments{
	\item{cnolist}{
		A list containing the experimental design and data.
	}
 	\item{model}{
		The logic model to be simulated.
	}
	\item{ode_parameters}{
		A list with the ODEs parameter information. Obtained with \code{\link{createLBodeContPars}}.
	}
	\item{indices}{
		Indices to map data in the model. Obtained with indexFinder function from CellNOptR. 
	}
	\item{time}{
		An integer with the index of the time point to start the simulation. Default is 1.
	}
	\item{verbose}{
		A logical value that triggers a set of comments.
	}
	 \item{transfer_function}{
		The type of used transfer. Use 1 for no transfer function, 2 for Hill function and
		3 for normalized Hill function.
	}
 	\item{reltol}{
		Relative Tolerance for numerical integration.
	}
	\item{atol}{
		Absolute tolerance for numerical integration.
	}
	\item{maxStepSize}{
		The maximum step size allowed to ODE solver.
	}
	\item{maxNumSteps}{
		The maximum number of internal steps between two points being sampled before
		the solver fails.
	}
 	\item{maxErrTestsFails}{
		Specifies the maximum number of error test failures permitted in attempting
		 one step.
	}
 	\item{nan_fac}{
		A penalty for each data point the model is not able to simulate. We recommend higher than
		0 and smaller that 1.
	}
}
	
\details{
	Check \code{\link{CellNOptR}} for details about the cnolist and the model format.
	For more details in the configuration of the ODE solver check the CVODES manual. 
}

\value{
	Returns a function to evaluate the model fitness. This function receives a continuous parameter
	vector.
}

\author{
	David Henriques, Thomas Cokelaer
}
%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
	\code{\link{CellNOptR}}
	\code{\link{createLBodeContPars}}
}

\examples{
library(CNORode)
data("ToyCNOlist",package="CNORode");
data("ToyModel",package="CNORode");
data("ToyIndices",package="CNORode");

ode_parameters=createLBodeContPars(model,random=TRUE);
minlp_obj_function=getLBodeMINLPObjFunction(cnolistCNORodeExample, model,ode_parameters,indices);

n_int_vars=dim(model$interMat)[2];
x_int=round(runif(n_int_vars))
x_cont=ode_parameters$parValues;
x=c(x_cont,x_int);
f=minlp_obj_function(x);
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.

\name{minlpLBodeSSm}

\alias{minlpLBodeSSm}
%- Also NEED an '\alias' for EACH other topic documented here.

\title{
	Search for the best combination of continuous parameters and logic gates.
}

\description{
	This function uses essR to search for the best set of continuous parameters and model structure. 
	The objective function is the same as the one provided by \code{\link{getLBodeMINLPObjFunction}}.
}

\usage{
	minlpLBodeSSm(cnolist, model, ode_parameters = NULL, int_x0=NULL, indices = NULL, maxeval = Inf,
	maxtime = 100, ndiverse = NULL, dim_refset = NULL, local_solver = NULL, time = 1, 
	verbose = 0, transfer_function = 3, reltol = 1e-04, atol = 0.001, maxStepSize = Inf,
	maxNumSteps = 1e+05, maxErrTestsFails = 50, nan_fac = 1)
}

%- maybe also 'usage' for other objects documented here.
\arguments{
 	\item{cnolist}{
		A list containing the experimental design and data.
	}
 	\item{model}{
		The logic model to be simulated.
	}
	\item{ode_parameters}{
		A list with the ODEs parameter information. Obtained 	with \code{\link{createLBodeContPars}}.
	}
	\item{int_x0}{
		Vector with initial solution for integer parameters.
	}
	\item{indices}{
		Indices to map data in the model. Obtained with indexFinder function from CellNOptR. 
	}
	\item{maxeval}{
		Maximum number of evaluation in the optimization procedure.
	}
	\item{maxtime}{
		Maximum number of evaluation spent in optimization procedure.
	}
	\item{ndiverse}{
		Duration of the optinisation procedure.
	}
	\item{dim_refset}{
		Number of diverse initial solutions.
	}
	\item{local_solver}{
		Local solver to be used in SSm.
	}
	\item{time}{
		An integer with the index of the time point to start the simulation. Default is 1.
	}
	\item{verbose}{
		A logical value that triggers a set of comments.
	}
 	\item{transfer_function}{
		The type of used transfer. Use 1 for no transfer function, 2 for Hill function and
		for normalized Hill function.
	}
	\item{reltol}{
		Relative Tolerance for numerical integration.
	}
	\item{atol}{
		Absolute tolerance for numerical integration.
	}
  	\item{maxStepSize}{
		The maximum step size allowed to ODE solver.
	}
	\item{maxNumSteps}{
		The maximum number of internal steps between two points being sampled before
		the solver fails.
	}
	\item{maxErrTestsFails}{
		Specifies the maximum number of error test failures permitted in attempting
		one step.
	}
  	\item{nan_fac}{
		A penalty for each data point the model is not able to simulate. We recommend higher than
		0 and smaller that 1.
	}
}

\details{
	Check \code{\link{CellNOptR}} for details about the cnolist and the model format.
	For more details in the configuration of the ODE solver check the CVODES manual. 
}

\value{
	\item{LB_n}{
		A numeric value to be used as lower bound for all parameters of type n.
	}
  	\item{LB_k}{
		A numeric value to be used as lower bound for all parameters of type k.
	}
	\item{LB_tau}{
		A numeric value to be used as lower bound for all parameters of type tau.
	}
	\item{UB_n}{
		A numeric value to be used as upper bound for all parameters of type n.
	}
	\item{UB_k}{
		A numeric value to be used as upper bound for all parameters of type k.
	}
	\item{UB_tau}{
		A numeric value to be used as upper bound for all parameters of type tau. 
	}
	\item{default_n}{
		The default parameter to be used for every parameter of type n. 
	}
  	\item{default_k}{
		The default parameter to be used for every parameter of type k. 
	}
  	\item{default_tau}{
		The default parameter to be used for every parameter of type tau. 
	}
	\item{LB_in}{
		An array with the the same length as ode_parameters$parValues with lower bounds
		for each specific parameter. 
	}
	\item{UB_in}{
		An array with the the same length as ode_parameters$parValues with upper bounds
		for each specific parameter.
	}
	\item{opt_n}{
		Add all parameter n to the index of parameters to be fitted.
	}
	\item{opt_k}{
		Add all parameter k to the index of parameters to be fitted.
	}
	\item{opt_tau}{
		Add all parameter tau to the index of parameters to be fitted.
	}
  	\item{random}{
		A logical value that determines that a random solution is for the parameters 
		to be optimised.
	}
	\item{model}{
		The best fitting found model structure.
	}
	\item{smm_results}{
		A list containing the information provided by the nonlinear optimization solver.
	}
}

\author{
	David Henriques, Thomas Cokelaer
}

%% ~Make other sections like Warning with \section{Warning }{....} ~
\seealso{
	\code{\link{CellNOptR}}
	\code{\link{createLBodeContPars}}
	\code{essR}
}

\examples{

\dontrun{
data("ToyCNOlist",package="CNORode");
data("ToyModel",package="CNORode");
data("ToyIndices",package="CNORode");
	
ode_parameters=createLBodeContPars(model,random=TRUE);

#Visualize initial solution
simulatedData=plotLBodeFitness(cnolistCNORodeExample, model,ode_parameters,indices=indices)
ode_parameters=minlpLBodeSSm(cnolistCNORodeExample, model,ode_parameters);

model=ode_parameters$model;

#Visualize fitted solution
simulatedData=plotLBodeFitness(cnolistCNORodeExample, model,indices=indices);
}

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/crossvalidateODE.R
\name{crossvalidateODE}
\alias{crossvalidateODE}
\title{Crossvalidate ODE model}
\usage{
crossvalidateODE(
  CNOlist,
  model,
  nfolds = 10,
  foldid = NULL,
  type = "datapoint",
  parallel = FALSE,
  ode_parameters = NULL,
  paramsSSm = NULL,
  method = "essm"
)
}
\arguments{
\item{CNOlist}{Cnolist which contains all the experiments}

\item{model}{a model prepared for the training}

\item{nfolds}{number of folds - default is 10. Although nfolds can be as large as the sample size (leave-one-out CV),
it is not recommended for large datasets.}

\item{foldid}{an optional vector of values between `1` and `nfold` identifying what fold each observation is in. If supplied, `nfold` can be missing.}

\item{type}{define the way to do the crossvalidation. The default is 
`type="datapoint"`, which assigns the data randomly into folds. 
The option `type="experiment"` uses whole experiments for crossvalidation 
(all data corresponding to a cue combination). The `type=observable` uses the
subset of nodes across all experiments for crossvalidation.}

\item{parallel}{use for parallel execution, requires the doParallel package}

\item{ode_parameters}{list of fitted logic ODE parameter}

\item{paramsSSm}{parameters for the SSm optimizer for running the optimization in crossvalidation}

\item{method}{Selection of optimization method: only "ga" or "essm" arguments are accepted}
}
\description{
k-fold crossvalidation for logic ODE model
}
\details{
Does a k-fold cross-validation for logic ODE CellNOpt models. In k-iterations a 
fraction of the data is eliminated from the CNOlist. The model is trained on the 
remaining data and then the model predicts the held-out data. Then the prediction
accuracy is reported for each iteration.
}
\seealso{
\code{\link{parEstimationLBode}}
}

\name{createLBodeContPars}

\alias{createLBodeContPars}
%- Also NEED an '\alias' for EACH other topic documented here.

\title{
	Create a list with ODE parameter information needed to perform parameter estimation
}

\description{
	Creates a list with the continuous parameters  to simulate the model,
	upper and lower bounds for the parameter estimation, parameters names, indices of
	the parameters and other information.
}

\usage{
	createLBodeContPars(model, LB_n = 1, LB_k = 0.1, LB_tau = 0.01, 
	UB_n = 5, UB_k = 0.9, UB_tau = 10, default_n = 3, default_k = 0.5,
	default_tau = 1, LB_in = c(), UB_in = c(), opt_n = TRUE, opt_k = TRUE,
	opt_tau = TRUE, random = FALSE)
}

%- maybe also 'usage' for other objects documented here.
\arguments{
 	\item{model}{
		The logic model to be simulated.
	}
	\item{LB_n}{
		A numeric value to be used as lower bound for all parameters of type n.
	}
	\item{LB_k}{
		A numeric value to be used as lower bound for all parameters of type k.
	}
	\item{LB_tau}{
	A numeric value to be used as lower bound for all parameters of type tau.
	}
	\item{UB_n}{
		A numeric value to be used as upper bound for all parameters of type n.
	}
	\item{UB_k}{
		A numeric value to be used as upper bound for all parameters of type k.
	}
	\item{UB_tau}{
		A numeric value to be used as upper bound for all parameters of type tau. 
	}	
  	\item{default_n}{
		The default parameter to be used for every parameter of type n. 
	}
 	\item{default_k}{
		The default parameter to be used for every parameter of type k. 
	}
	\item{default_tau}{
		The default parameter to be used for every parameter of type tau. 
	}
	 \item{LB_in}{
		An array with the the same length as ode_parameters$parValues with lower bounds
		for each specific parameter. 
	}
	\item{UB_in}{
		An array with the the same length as ode_parameters$parValues with upper bounds
		for each specific parameter.
	}
	\item{opt_n}{
		Add all parameter n to the index of parameters to be fitted.
	}
	\item{opt_k}{
		Add all parameter k to the index of parameters to be fitted.
	}
	\item{opt_tau}{
		Add all parameter tau to the index of parameters to be fitted.
	}
	\item{random}{
		 logical value that determines that a random solution is for the parameters 
		to be optimized.
	}
}

\value{
	\item{parNames}{An array containing the names of the parameters.}
	\item{parValues}{An array containing the values of the parameters, in the same order as the names.}
	\item{index_opt_pars}{An array containing the indexes for the parameters to be fitted.}
	\item{index_n}{An array containing the indexes of the parameters of type n.}
	\item{index_k}{An array containing the indexes of the parameters of type k.}
	\item{index_tau}{An array containing the indexes of the parameters of type tau.}
	\item{LB}{An array containing the lower bound for each parameter.}
	\item{UB}{An array containing the upper bound for each parameter.}
}

\author{
	David Henriques, Thomas Cokelaer
}

\examples{
	library(CNORode)
	data("ToyCNOlist",package="CNORode");
	data("ToyModel",package="CNORode");
	data("ToyIndices",package="CNORode");
	ode_parameters=createLBodeContPars(model, opt_n=FALSE,default_n=2,
	random=TRUE,LB_k=0.25,UB_k=0.8,LB_tau=0.01,UB_tau=10);
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.

\name{simdata2cnolist}

\alias{simdata2cnolist}

\title{
	converts output of getLBodeModelSim to cnolist
}

\description{
	This function converts the simulated data returned by getLBodeModelSim into
    a valid CNOlist data structure.
}

\usage{
	simdata2cnolist(sim_data, cnolist, model)
}

\arguments{
    \item{sim_data}{
        structure returned by getLBodeModelSim
    }
	\item{cnolist}{
		A list containing the experimental design and data. 
	}
	\item{model}{
		The logic model to be simulated.
	}
}


\value{
	a CNOlist
}

\author{
	Thomas Cokelaer
}
%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
	\code{\link{CellNOptR}}
	\code{\link{createLBodeContPars}}
}

\examples{
data('ToyCNOlist',package='CNORode');
data('ToyModel',package='CNORode');
data('ToyIndices',package='CNORode');
simdata = getLBodeModelSim(cnolistCNORodeExample, model,indices=indices)
cnolist = simdata2cnolist(simdata, cnolistCNORodeExample, model)

cnolist = simdata2cnolist(simdata, cnolistCNORodeExample, model)
}

\name{getStates}

\alias{getStates}
%- Also NEED an '\alias' for EACH other topic documented here.

\title{
	Find which species in the model are states.
}

\description{
	Receives an adjacency matrix (model$interMat from CellNoptR) and finds which species are states (i.e. not inputs).
}

\usage{
	getStates(adjacency)
}
%- maybe also 'usage' for other objects documented here.

\arguments{
 	\item{adjacency}{
		An adjacency matrix from the model.
	}
}

\value{
	A numeric vector with 0's for positions which are states and 1's  for positions which are.
}

\note{
	For internal use of CNORode.
}

\author{
	David Henriques, Thomas Cokelaer
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
	\code{\link{incidence2Adjacency}}
}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{states}
\keyword{adjacency}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runCNORode.R
\name{runCNORode}
\alias{runCNORode}
\title{runCNORode}
\usage{
runCNORode(
  model,
  data,
  compression = TRUE,
  results_folder = "CNORode_results",
  cutNONC = TRUE,
  expansion = FALSE,
  LB_n = 1,
  LB_k = 0.1,
  LB_tau = 0.01,
  UB_n = 5,
  UB_k = 0.9,
  UB_tau = 10,
  default_n = 3,
  default_k = 0.5,
  default_tau = 1,
  opt_n = TRUE,
  opt_k = TRUE,
  opt_tau = TRUE,
  random = TRUE,
  maxeval = 1e+05,
  maxtime = 60,
  transfer_function = 3,
  nan_fac = 1,
  lambda_tau = 0,
  lambda_k = 0
)
}
\arguments{
\item{model}{A filename of prior knowledge network (PKN) in the SIF format}

\item{data}{A measurement filename in the MIDAS format}

\item{compression}{compress the prior knowledge network (TRUE), see \code{\link{preprocessing}}}

\item{results_folder}{results folder for the analysis.}

\item{cutNONC}{cut non-observable non-controllable node from PKN (TRUE), see \code{\link{preprocessing}}}

\item{expansion}{expand OR gates in the PKN (FALSE), see \code{\link{preprocessing}}}

\item{LB_n}{lower bound on parameter n, see \code{\link{createLBodeContPars}}}

\item{LB_k}{lower bound on parameter k, see \code{\link{createLBodeContPars}}}

\item{LB_tau}{lower bound on parameter tau, see \code{\link{createLBodeContPars}}}

\item{UB_n}{upper bound on parameter n, see \code{\link{createLBodeContPars}}}

\item{UB_k}{upper bound on parameter k, see \code{\link{createLBodeContPars}}}

\item{UB_tau}{upper bound on parameter tau, see \code{\link{createLBodeContPars}}}

\item{default_n}{default value of parameter n, see \code{\link{createLBodeContPars}}}

\item{default_k}{default value of parameter k, see \code{\link{createLBodeContPars}}}

\item{default_tau}{default value of parameter tau, see \code{\link{createLBodeContPars}}}

\item{opt_n}{should parameter n be optimised, see \code{\link{createLBodeContPars}}}

\item{opt_k}{should parameter k be optimised, see \code{\link{createLBodeContPars}}}

\item{opt_tau}{should parameter tau be optimised, see \code{\link{createLBodeContPars}}}

\item{random}{initial parameter vector generation (TRUE: random, FALSE: half of the LB-UB)}

\item{maxeval}{maximum number of funciton evaluations in the optimisation, see \code{\link{parEstimationLBodeSSm}}}

\item{maxtime}{maximum CPU time (in seconds) spent on optimisation before calling final refinement, see \code{\link{parEstimationLBodeSSm}}}

\item{transfer_function}{trandfer function types represented by the edges, see \code{\link{parEstimationLBodeSSm}}}

\item{nan_fac}{penalty for NA simulations, see \code{\link{parEstimationLBodeSSm}}}

\item{lambda_tau}{regularisation penalty for tau parameters, see \code{\link{parEstimationLBodeSSm}}}

\item{lambda_k}{regularisation penalty for k parameters for optimisation, see \code{\link{parEstimationLBodeSSm}}}
}
\description{
A one-line wrapper of the CNORode pipeline
}
\examples{
\dontrun{
model = system.file("extdata", "ToyModelMMB_FeedbackAnd.sif",package="CNORode")
data = system.file("extdata", "ToyModelMMB_FeedbackAnd.csv", package="CNORode")
res = runCNORode(model,data,results_folder = "./results")
}
}

\name{simulate}

\alias{simulate}

\title{
	Simulate value signals a CNO list With Logic-Based ODEs.
}

\description{
	This function receives a set of inputs, namely the cnolist and the model and
	returns a list with the same size of the cnolist$valueSignals. 
}

\usage{
	simulate(cnolist, model, ode_parameters=NULL, indices=NULL,
	adjMatrix=NULL, time=1, verbose=0, transfer_function=3,
	reltol=1e-04, atol=0.001, maxStepSize=Inf, maxNumSteps=1e+05, 
	maxErrTestsFails=50)
}

	\arguments{
  	\item{cnolist}{
  		A list containing the experimental design and data.
	}
 	 \item{model}{
	A list with the ODEs parameter information. Obtained with \code{\link{createLBodeContPars}}.
	}
  	\item{ode_parameters}{
		A list with the ODEs parameter information. Obtained with makeParameterList function.
	}
	  \item{indices}{
		Indices to map data in the model. Obtained with indexFinder function from CellNOptR. 
	}
	\item{adjMatrix}{
		The adjacency matrix. Recomputed if not provided
	}
  	\item{time}{
		An integer with the index of the time point to start the simulation. Default is 1.
	}
	\item{verbose}{
		A logical value that triggers a set of comments.
	}
	\item{transfer_function}{
		The type of used transfer. Use 1 for no transfer function, 2 for Hill function and
		3 for normalized Hill function.
	}
	\item{reltol}{
		Relative Tolerance for numerical integration.
	}
 	\item{atol}{
		Absolute tolerance for numerical integration.
	}
	\item{maxStepSize}{
		The maximum step size allowed to ODE solver.
	}
	\item{maxNumSteps}{
		The maximum number of internal steps between two points being sampled before
		the solver fails.
	}
  	\item{maxErrTestsFails}{
		Specifies the maximum number of error test failures permitted in attempting
		one step.
	}
}

\details{
	Check \code{\link{CellNOptR}} for details about the cnolist and the model format.
	For more details in the configuration of the ODE solver check the CVODES manual. 
}

\value{
	Returns a list with simulated data that has the same structure as the cnolist$valueSignals.
	One matrix for each time-point.
}

\author{
	David Henriques, Thomas Cokelaer
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
	\code{\link{CellNOptR}}
	\code{\link{parEstimationLBode}}
	\code{\link{parEstimationLBodeSSm}}
}

\examples{
library(CNORode)
data("ToyCNOlist",package="CNORode");
data("ToyModel",package="CNORode");
data("ToyIndices",package="CNORode");
dataSimulation = simulate(cnolistCNORodeExample, model,indices=indices);
}

\name{parEstimationLBodeGA}

\alias{parEstimationLBodeGA}
%- Also NEED an '\alias' for EACH other topic documented here.

\title{
	Perform parameter estimation using a genetic algorithm (package genalg).
}

\description{
	This function uses a genetic algorithm (package genalg) to perform parameter estimation. The objective function
	is the same as the one provided by \code{\link{getLBodeContObjFunction}}.
}

\usage{
	parEstimationLBodeGA(cnolist, model, ode_parameters = NULL, indices = NULL, mutationChance = NA, popSize = 200, iters = 100, 
		elitism = NA, time = 1, monitor = TRUE, verbose = 0, transfer_function = 3, reltol = 1e-04, 
		atol = 0.001, maxStepSize = Inf, maxNumSteps = 1e+05, maxErrTestsFails = 50, nan_fac = 1)
}
%- maybe also 'usage' for other objects documented here.

\arguments{
 	\item{cnolist}{
		A list containing the experimental design and data.
	}
	\item{model}{
		The logic model to be simulated.
	}
	 \item{ode_parameters}{
		A list with the ODEs parameter information. Obtained with \code{\link{createLBodeContPars}}.
	}
  	\item{indices}{
		Indices to map data in the model. Obtained with indexFinder function from CellNOptR. 
	}
  	\item{mutationChance}{
		the chance that a gene in the chromosome mutates. By default 1/(size+1). It affects
		the convergence rate and the probing of search space: a low chance results
		in quicker convergence, while a high chance increases the span of the search
		space.
	}	
	\item{popSize}{
		the population size.	
	}
	\item{iters}{
		the number of iterations.
	}
	\item{elitism}{
		the number of chromosomes that are kept into the next generation. By default is
		about 20\% of the population size
	}
	\item{time}{
		An integer with the index of the time point to start the simulation. Default is 1.
	}
	 \item{monitor}{
		 If TRUE a plot will be generated to monitor the objective function
	}
	\item{verbose}{
		A logical value that triggers a set of comments.
	}
	\item{transfer_function}{
		The type of used transfer. Use 1 for no transfer function, 2 for Hill function and
		3 for normalized Hill function.
	}
	\item{reltol}{
		Relative Tolerance for numerical integration.
	}
  	\item{atol}{
		Absolute tolerance for numerical integration.
	}
	\item{maxStepSize}{
		The maximum step size allowed to ODE solver.
	}
	\item{maxNumSteps}{
		The maximum number of internal steps between two points being sampled before
		the solver fails.
	}
	\item{maxErrTestsFails}{
		Specifies the maximum number of error test failures permitted in attempting
	 	one step.
	}
	\item{nan_fac}{
		A penalty for each data point the model is not able to simulate. We recommend higher than
		0 and smaller that 1.
	}
}

\value{
	\item{LB_n}{
		A numeric value to be used as lower bound for all parameters of type n.
	}
	\item{LB_k}{
		A numeric value to be used as lower bound for all parameters of type k.
	}
	\item{LB_tau}{
		A numeric value to be used as lower bound for all parameters of type tau.
	}
	\item{UB_n}{
		A numeric value to be used as upper bound for all parameters of type n.
	}
 	\item{UB_k}{
		A numeric value to be used as upper bound for all parameters of type k.
	}
  	\item{UB_tau}{
		A numeric value to be used as upper bound for all parameters of type tau. 
	}
  	\item{default_n}{
		The default parameter to be used for every parameter of type n. 
	}
 	 \item{default_k}{
		The default parameter to be used for every parameter of type k. 
	}
	\item{default_tau}{
		The default parameter to be used for every parameter of type tau. 
	}
	\item{LB_in}{
		An array with the the same length as ode_parameters$parValues with lower bounds
		for each specific parameter. 
	}
	\item{UB_in}{
		An array with the the same length as ode_parameters$parValues with upper bounds
		for each specific parameter.
	}
	\item{opt_n}{
		Add all parameter n to the index of parameters to be fitted.
	}
	\item{opt_k}{
		Add all parameter k to the index of parameters to be fitted.
	}
	\item{opt_tau}{
		Add all parameter tau to the index of parameters to be fitted.
	}
	\item{random}{
		A logical value that determines that a random solution is for the parameters 
		to be optimized.
	}
	\item{res}{
		A list containing the information provided by the nonlinear optimization solver (genalg).
	}
}


\author{
	David Henriques, Thomas Cokelaer
}
%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
	\code{\link{CellNOptR}}
	\code{\link{createLBodeContPars}}
	\code{\link{rbga}}
}

\examples{
data("ToyCNOlist",package="CNORode");
data("ToyModel",package="CNORode");
data("ToyIndices",package="CNORode");
	
ode_parameters=createLBodeContPars(model,random=TRUE);
#Visualize intial simulation
#simulatedData=plotLBodeFitness(cnolistCNORodeExample, model,ode_parameters,indices=indices)

ode_parameters=parEstimationLBodeGA(cnolistCNORodeExample,model,ode_parameters=ode_parameters,
indices=indices,maxStepSize=1,atol=1e-3,reltol=1e-5,transfer_function=2,popSize=10,iter=40);

#Visual solution after optimization
simulatedData=plotLBodeFitness(cnolistCNORodeExample, model,indices=indices,ode_parameters=ode_parameters);
}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{genetic}
\keyword{algorithm}% __ONLY ONE__ keyword per line
\keyword{CellNOptR}
\keyword{logic}
\keyword{model}

\name{getLBodeSimFunction}

\alias{getLBodeSimFunction}

\title{
	Get a function to simulate a logic based ODE model.
}

\description{
	This function is internally used by CNORode to configure the simulation function with default arguments.
}

\usage{
	getLBodeSimFunction(cnolist1, model1, adjMatrix1, indices1, odeParameters1,
	time1 = 1, verbose1 = 0, transfer_function1 = 3, reltol1 = 1e-04, atol1 = 0.001,
	maxStepSize1 = Inf, maxNumSteps1 = 1e+05, maxErrTestsFails1 = 50)
}

%- maybe also 'usage' for other objects documented here.
\arguments{
 	\item{cnolist1}{
		A list containing the experimental design and data.
	}
	\item{model1}{
		The logic model to be simulated.
	}
 	\item{adjMatrix1}{
		An adjacency matrix from the model.
	}
	\item{indices1}{
		Indices to map data in the model. Obtained with indexFinder function from CellNOptR. 
	}
	\item{odeParameters1}{
		A list with the ODEs parameter information. Obtained with \code{\link{createLBodeContPars}}.
	}	
	\item{time1}{
		An integer with the index of the time point to start the simulation. Default is 1.
	}
	\item{verbose1}{
		A logical value that triggers a set of comments.
	}
	\item{transfer_function1}{
		The type of used transfer. Use 1 for no transfer function, 2 for Hill function and
		3 for normalized Hill function.
	}
	\item{reltol1}{
		Relative Tolerance for numerical integration.
	}
	\item{atol1}{
		Absolute tolerance for numerical integration.
	}
	\item{maxStepSize1}{
		The maximum step size allowed to ODE solver.
	}
	\item{maxNumSteps1}{
		The maximum number of internal steps between two points being sampled before
		the solver fails.
	}
	\item{maxErrTestsFails1}{
		Specifies the maximum number of error test failures permitted in attempting
		one step.
	}
}

\value{
	A function that returns a simulated model.
}

\author{
	David Henriques, Thomas Cokelaer
}

\note{
	This function is for CNORode internal use.
}

\seealso{
	\code{\link{CellNOptR}}
	\code{\link{CNORode}}
}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{CVODES}

\name{incidence2Adjacency}

\alias{incidence2Adjacency}

\title{
	Convert an incidence matrix into an adjacency matrix.
}

\description{
	Convert the incidence matrix (model representation of CellNoptR) into an adjacency matrix. Denotes the inputs/output relationships.
}

\usage{
	incidence2Adjacency(model)
}

\arguments{
 	\item{model}{
		Model from CellNoptR.
	}
}

\value{
	Directed Adjacency matrix of size n_species by n_species. 
}

\note{
	For internal use of CNORode.
}

\author{
	David Henriques, Thomas Cokelaer
}

\seealso{
	\code{\link{CellNOptR}}
}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{incidence}
\keyword{adjacency}
\keyword{matrix}

\name{defaultParametersSSm}

\alias{defaultParametersSSm}
%- Also NEED an '\alias' for EACH other topic documented here.

\title{
	Create default options to perform parameter estimation with scatter search meta-heuristic.
}

\description{
	This function returns a list with several arguments for performing parameter estimation with scatter search meta-heuristic 
	algorithm from the package essR.
}

\usage{
	defaultParametersSSm()
}

\value{
	\item{maxeval}{Inf}
	\item{maxtime}{100}
	\item{ndiverse}{NULL}
	\item{dim_refset}{NULL}
	\item{local_solver}{NULL}
	\item{verbose}{0}
	\item{transfer_function}{3}
	\item{reltol}{1e-04}
	\item{atol}{0.001}
	\item{maxStepSize}{Inf}
	\item{maxNumSteps}{1e+05}
	\item{maxErrTestsFails}{50}
	\item{nan_fac}{1}	
	\item{lambda_tau}{0}	
	\item{lambda_k}{0}	
	\item{bootstrap}{0}	
	\item{SSpenalty_fac}{0}	
	\item{SScontrolPenalty_fac}{0}	
	\item{boot_seed}{sample(1:10000,1)}	
}

\author{
	David Henriques, Thomas Cokelaer, Federica Eduati
}

\seealso{
	\code{\link{CellNOptR}}
	\code{\link{parEstimationLBode}}
	\code{\link{parEstimationLBodeSSm}}
}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{default}
\keyword{SSm}
\keyword{essR}

\name{plotLBodeModelSim}

\alias{plotLBodeModelSim}
%- Also NEED an '\alias' for EACH other topic documented here.

\title{
 	Simulate the model and plot the obtained with the different experimental conditions.
}

\description{
	Plots the simulated values of the logic based ODE model.
	Only dynamic states are plotted, i.e. those that are not inputs. 
	a blue line. Additionally this functions returns the the simulated
	values.
}

\usage{
	plotLBodeModelSim(cnolist, model, ode_parameters = NULL, indices = NULL, 
	adjMatrix = NULL, timeSignals=NULL, time = 1, verbose = 0, transfer_function = 3,
	reltol = 1e-04, atol = 0.001, maxStepSize = Inf, maxNumSteps = 1e+05,
	maxErrTestsFails = 50, large = FALSE, nsplit = 4, show = TRUE)
}
%- maybe also 'usage' for other objects documented here.

\arguments{
	\item{cnolist}{
		A list containing the experimental design and data.
	}
	\item{model}{
		The logic model to be simulated.
	}
  \item{ode_parameters}{
	A list with the ODEs parameter information. Obtained with \code{\link{createLBodeContPars}}.
	}
	\item{indices}{
		Indices to map data in the model. Obtained with indexFinder function from CellNOptR. 
	}
	\item{adjMatrix}{
		Model representation in the form of an adjacency matrix. When not provided
		will be automatically computed based in the model.
	}
	\item{timeSignals}{
		An array containing a different timeSignals. If you use this argument, it will also
		modify the dimensions from valueSignals.
	}
	\item{time}{
		An integer with the index of the time point to start the simulation. Default is 1.
	}
	\item{verbose}{
		A logical value that triggers a set of comments.
	}
	\item{transfer_function}{
		The type of used transfer. Use 1 for no transfer function, 2 for Hill function and
		3 for normalized Hill function.
	}	
	\item{reltol}{
		Relative Tolerance for numerical integration.
	}
  	\item{atol}{
		Absolute tolerance for numerical integration.
	}
	\item{maxStepSize}{
		The maximum step size allowed to ODE solver.
	}
	\item{maxNumSteps}{
		The maximum number of internal steps between two points being sampled before
		the solver fails.
	}
	\item{maxErrTestsFails}{
		Specifies the maximum number of error test failures permitted in attempting
		one step.
	}
	\item{large}{
		Boolean variable defining if the plot should split into several subplots.
	}
 	\item{nsplit}{
		In case the large plot options is selected define how many subplots will exist. Default is 4.
	}
	\item{show}{
		Boolean variable defining if we shold plot the CNOlist object.
	}
}

\value{
	Returns a list with simulated Model values. One matrix of size number of species by number of
	 experimental conditions for each time-point.
}

\author{
	David Henriques, Thomas Cokelaer
}

\seealso{
	\code{\link{CellNOptR}}
	\code{\link{createLBodeContPars}}
}

\examples{
    	library(CNORode)
	data("ToyCNOlist",package="CNORode");
	data("ToyModel",package="CNORode");
	data("ToyIndices",package="CNORode");
	modelSimulation=plotLBodeModelSim(cnolistCNORodeExample, model,indices=indices);
}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.

\name{defaultParametersGA}

\alias{defaultParametersGA}

\title{
	Create default options to perform parameter estimation with a genetic algorithm.
}

\description{
	This function returns a list with several arguments for performing parameter estimation with the genetic 
	algorithm from the package genalg.
}
\usage{
	defaultParametersGA()
}
%- maybe also 'usage' for other objects documented here.

\value{
	\item{mutationChance}{NA}
	\item{popSize}{200}
	\item{iters}{100}
	\item{elitism}{NA}
	\item{time}{1}
	\item{monitor}{TRUE}
	\item{verbose}{0}
	\item{transfer_function}{3}
	\item{reltol}{1e-04}
	\item{atol}{0.001}
	\item{maxStepSize}{Inf}
	\item{maxNumSteps}{1e+05}
	\item{maxErrTestsFails}{50}
	\item{nan_fac = 1}{0}
}

\author{
	David Henriques, Thomas Cokelaer
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
	\code{\link{CellNOptR}}
	\code{\link{parEstimationLBode}}
	\code{\link{parEstimationLBodeGA}}
}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.

\keyword{default }
\keyword{parameters}
\keyword{genetic}
\keyword{algorithm}

\name{getLBodeContObjFunction}

\alias{getLBodeContObjFunction}

\title{
	Returns the objective function to perform parameter estimation.
}

\description{
	This function configures returns the objective function that can be used to evaluate
	the fitness of a logic based ODE model using a particular set of parameters.
	This function can be particularly useful if you are planing to couple a nonlinear
	optimization solver. The returned value of the objective function corresponds to 
	the mean squared value normalized by the number of data points.
}

\usage{
	getLBodeContObjFunction(cnolist, model, ode_parameters, indices=NULL, time = 1, 
	verbose = 0, transfer_function = 3, reltol = 1e-04, atol = 0.001, maxStepSize = Inf, 
	maxNumSteps = 1e+05, maxErrTestsFails = 50, nan_fac = 1, lambda_tau=0, lambda_k=0,
	bootstrap=F, SSpenalty_fac=0, SScontrolPenalty_fac=0, boot_seed=sample(1:10000,1))
}

%- maybe also 'usage' for other objects documented here.
\arguments{
 	\item{cnolist}{
		A list containing the experimental design and data.
	}
	\item{model}{
		The logic model to be simulated.
	}
 	\item{ode_parameters}{
		A list with the ODEs parameter information. Obtained with \code{\link{createLBodeContPars}}.
	}
	\item{indices}{
		Indices to map data in the model. Obtained with indexFinder function from CellNOptR. 
	}
	\item{time}{
		An integer with the index of the time point to start the simulation. Default is 1.
	}
	\item{verbose}{
		A logical value that triggers a set of comments.
	}
	\item{transfer_function}{
		The type of used transfer. Use 1 for no transfer function, 2 for Hill function and
		3 for normalized Hill function.
	}
	\item{reltol}{
		Relative Tolerance for numerical integration.
	}
	\item{atol}{
		Absolute tolerance for numerical integration.
	}
	\item{maxStepSize}{
		The maximum step size allowed to ODE solver.
	}
	\item{maxNumSteps}{
		The maximum number of internal steps between two points being sampled before
		the solver fails.
	}
	\item{maxErrTestsFails}{
		Specifies the maximum number of error test failures permitted in attempting
		one step.
	}
  	\item{nan_fac}{
		A penalty for each data point the model is not able to simulate. We recommend higher than
		0 and smaller that 1.
	}
	  \item{lambda_tau}{
		Tunable regularisation parameters to penalise L1-norm of parameters tau and induce sparsity.
		We recommend testing values between 0 and 100 (in log scale) to find best compromise between
		good fit and sparse model. Default =0, corresponding to no regularisation.
	}
	  \item{lambda_k}{
		Tunable regularisation parameters to penalise L1-norm of parameters k and induce sparsity.
		We recommend testing values between 0 and 100 (in log scale) to find best compromise between
		good fit and sparse model. Default =0, corresponding to no regularisation.
	}
	  \item{bootstrap}{
		If set to TRUE performs random sampling with replacement of the measurements used in the
		optimisation (to be run multiple times to get bootstrapped distribution of parameters).
		Default =FALSE, no bootstrapping.
	}
	  \item{SSpenalty_fac}{
		Penalty factor for penalising solutions which do not reach steady state. Default =0.
	}
	  \item{SScontrolPenalty_fac}{
		Penalty factor for penalising solutions for which the control (unperturbed) condition
		(assumed to be first row) does not reach steady state. Default =0.
	}
	  \item{boot_seed}{
		Seed used for random sampling if bootstrap=TRUE. Default chose random seed between 0 and 10000
	}
}

\details{
	Check \code{\link{CellNOptR}} for details about the cnolist and the model format.
	For more details in the configuration of the ODE solver check the CVODES manual. 
}

\value{
	Returns a function to evaluate the model fitness. This function receives a vector
	containing both continuous parameters and integer values representing which reactions
	should be kept in the model.
}

\author{
	David Henriques, Thomas Cokelaer, Federica Eduati
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
	\code{\link{CellNOptR}}
	\code{\link{createLBodeContPars}}
}

\examples{
 	library(CNORode)
	data("ToyCNOlist",package="CNORode");
	data("ToyModel",package="CNORode");
	data("ToyIndices",package="CNORode");
	
	ode_parameters=createLBodeContPars(model,random=TRUE);
	minlp_obj_function=getLBodeContObjFunction(cnolistCNORodeExample, model,ode_parameters,indices);
	
	x=ode_parameters$parValues;
	
	f=minlp_obj_function(x);
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
